@@ -22,7 +22,6 @@
 
 #include <string.h>
 #include <libgnomedb/libgnomedb.h>
-#include <libgnomedb/gnome-db-window.h>
 #include <glade/glade-init.h>
 #include <glade/glade-build.h>
 
@@ -43,29 +42,6 @@ gnomedb_dialog_find_internal_child (GladeXML *xml,
 	return NULL;
 }
 
-static GtkWidget *
-gnomedb_window_find_internal_child (GladeXML    *xml,
-				    GtkWidget   *parent,
-				    const gchar *childname)
-{
-        if (!strcmp (childname, "vbox")) {
-                GtkWidget *ret;
-
-                if ((ret = bonobo_window_get_contents (BONOBO_WINDOW (parent))))
-                        return ret;
-                else {
-                        GtkWidget *box;
-
-                        box = gtk_vbox_new (FALSE, 0);
-                        bonobo_window_set_contents (BONOBO_WINDOW (parent), box);
-
-                        return box;
-                }
-        }
-
-	return NULL;
-}
-
 /* this macro puts a version check function into the module */
 GLADE_MODULE_CHECK_INIT
 
