@@ -32,7 +32,6 @@
 #include "gnome-db-data-handler.h"
 
 struct _GnomeDbComboPrivate {
-	GnomeDbDict      *dict;
 	GnomeDbDataModel *model;
 	GnomeDbDataProxy *proxy;
 
@@ -115,6 +114,8 @@ gnome_db_combo_init (GnomeDbCombo *combo, GnomeDbComboClass *klass)
 	combo->priv = g_new0 (GnomeDbComboPrivate, 1);
 	combo->priv->model = NULL;
 	combo->priv->proxy = NULL;
+
+	gtk_combo_box_set_wrap_width (GTK_COMBO_BOX (combo), 0);
 }
 
 static void
@@ -201,27 +202,23 @@ gnome_db_combo_finalize (GObject *object)
 
 /**
  * gnome_db_combo_new
- * @dict: a #GnomeDbDict object, or %NULL
  *
  * Create a new GnomeDbCombo widget.
  *
  * Returns: the newly-created widget.
  */
 GtkWidget *
-gnome_db_combo_new (GnomeDbDict *dict)
+gnome_db_combo_new ()
 {
 	GnomeDbCombo *combo;
 
 	combo = g_object_new (GNOME_DB_TYPE_COMBO, NULL);
-	combo->priv->dict = dict; /* FIXME: what if dict is destroyed? */
 
-	gtk_combo_box_set_wrap_width (GTK_COMBO_BOX (combo), 0);
 	return GTK_WIDGET (combo);
 }
 
 /**
  * gnome_db_combo_new_with_model
- * @dict: a #GnomeDbDict object, or %NULL
  * @model: a #GdaDataModel object.
  * @n_cols: number of columns in the model to be shown
  * @cols_index: index of each column to be shown
@@ -231,13 +228,13 @@ gnome_db_combo_new (GnomeDbDict *dict)
  * Returns: the newly-created widget.
  */
 GtkWidget *
-gnome_db_combo_new_with_model (GnomeDbDict *dict, GdaDataModel *model, gint n_cols, gint *cols_index)
+gnome_db_combo_new_with_model (GdaDataModel *model, gint n_cols, gint *cols_index)
 {
 	GnomeDbCombo *combo;
 	
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
-	combo = GNOME_DB_COMBO (gnome_db_combo_new (dict)); 
+	combo = GNOME_DB_COMBO (gnome_db_combo_new ()); 
 	gnome_db_combo_set_model (GNOME_DB_COMBO (combo), GDA_DATA_MODEL (model), n_cols, cols_index);
 
 	return GTK_WIDGET (combo);
