@@ -120,7 +120,7 @@ GQuark gnome_db_constraint_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_constraint_get_type (void)
 {
 	static GType type = 0;
