@@ -135,7 +135,7 @@ GQuark gnome_db_custom_layout_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_custom_layout_get_type (void)
 {
 	static GType type = 0;
