@@ -60,7 +60,6 @@ static gboolean          data_proxy_iter_parent     (GtkTreeModel *tree_model, G
 
 static gint model_to_iter_numbers  (GnomeDbDataProxy *proxy, gint row);
 static gint iter_to_model_numbers  (GnomeDbDataProxy *proxy, gint row);
-static void re_compute_row_numbers (GnomeDbDataProxy *proxy);
 
 static void nullified_object_cb (GnomeDbBase *obj, GnomeDbDataProxy *proxy);
 #ifdef debug
@@ -93,7 +92,6 @@ enum
 typedef struct
 {
 	gint           model_row;    /* row index in the GnomeDbDataModel, -1 if new row */
-	gint           iter_row;     /* row index for the GtkTreeIter interface, always >= 0 */
 	gboolean       to_be_deleted;/* TRUE if row is to be deleted */
 	GSList        *modif_values; /* list of RowValue structures */
 	GdaValue     **orig_values;  /* array of the original GdaValues, indexed on the column numbers */
@@ -134,19 +132,26 @@ struct _GnomeDbDataProxyPrivate
 	GnomeDbDataSet    *info_data_set; /* used for columns attributes and initial values when a new row is created */
 
 	gint               model_nb_cols; /* = gda_data_model_get_n_columns (model) */
-	gint               model_nb_rows; /* = gda_data_model_get_n_rows (model) */
 	gboolean           autocommit;
 	gint               stamp;       /* Random integer to check whether an iter belongs to our model */
 
 	GSList            *all_modifs; /* list of RowModif structures, for memory management */
 	GSList            *new_rows;   /* list of RowModif, no data allocated in this list */
 
-	GHashTable        *iter_rows;  /* key = row number for the GtkTreeModel, value = RowModif */
+	GHashTable        *modif_rows;  /* key = row number for the GnomeDbDataModel, value = RowModif, NOT for new rows */
 
 	gint               internal_changes;
 	gboolean           model_refresh_pending;
 
 	gboolean           add_null_entry; /* artificially add a NULL entry at the beginning of the tree model */
+
+	guint              idle_add_event_source; /* !=0 when data rows are being added */
+
+	/* chunking: ONLY accounting for @model's rows, not the new rows */
+	gint               sample_first_row;
+	gint               sample_last_row;
+	gint               sample_size;
+	gint               current_nb_rows;
 };
 
 /*
@@ -163,9 +168,9 @@ static gint
 model_to_iter_numbers (GnomeDbDataProxy *proxy, gint row)
 {
 	if (proxy->priv->add_null_entry)
-		return row + 1;
+		return row - proxy->priv->sample_first_row + 1;
 	else
-		return row;
+		return row - proxy->priv->sample_first_row;
 }
 
 static gint
@@ -175,48 +180,36 @@ iter_to_model_numbers (GnomeDbDataProxy *proxy, gint row)
 		if (row == 0)
 			return -1;
 		
-		if (row < proxy->priv->model_nb_rows + 1)
-			return row - 1;
+		if (row < proxy->priv->current_nb_rows + 1)
+			return row + proxy->priv->sample_first_row - 1;
 		else
 			return -1;
 	}
 	else {
-		if (row < proxy->priv->model_nb_rows)
-			return row;
+		if (row < proxy->priv->current_nb_rows)
+			return row + proxy->priv->sample_first_row;
 		else
 			return -1;
 	}
 }
 
-/*
- * Should be called whenever a row in removed from the GtkTreeModel
- *
- * Ensures that the RowModif->iter_row are correct and in sunc with what's in @proxy->priv->iter_rows
- */
-static void
-re_compute_row_numbers (GnomeDbDataProxy *proxy)
+static RowModif *
+find_row_modif_for_iter (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 {
-	GSList *list = proxy->priv->new_rows;
-	RowModif *rm;
-	gint num;
-	gint oldnum, newnum;
+	RowModif *rm = NULL;
+	gint model_row;
 
-	num = g_slist_length (proxy->priv->new_rows) - 1;
-	while (list) {
-		rm = ROW_MODIF (list->data);
+	model_row = iter_to_model_numbers (proxy, GPOINTER_TO_INT (iter->user_data));
+	if (model_row >= 0)
+		rm = g_hash_table_lookup (proxy->priv->modif_rows, GINT_TO_POINTER (model_row));
+	else {
+		gint offset = proxy->priv->current_nb_rows + (proxy->priv->add_null_entry ? 1 : 0);
 
-		oldnum = rm->iter_row;
-		newnum = proxy->priv->model_nb_rows + num + (proxy->priv->add_null_entry ? 1 : 0);
-		if (oldnum != newnum) {
-			/*g_print ("renumbered %d to %d\n", oldnum, newnum);*/
-			rm->iter_row = newnum;
-			g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (oldnum));
-			g_hash_table_insert (proxy->priv->iter_rows, GINT_TO_POINTER (newnum), rm);
-		}
-
-		num--;
-		list = g_slist_next (list);
+		offset = GPOINTER_TO_INT (iter->user_data) - offset;
+		rm = g_slist_nth_data (proxy->priv->new_rows, offset);
 	}
+
+	return rm;
 }
 
 /*
@@ -284,7 +277,7 @@ GQuark gnome_db_data_proxy_error_quark (void)
         return quark;
 }
 
-guint
+GType
 gnome_db_data_proxy_get_type (void)
 {
 	static GType type = 0;
@@ -363,12 +356,22 @@ static void
 gnome_db_data_proxy_init (GnomeDbDataProxy *proxy)
 {
 	proxy->priv = g_new0 (GnomeDbDataProxyPrivate, 1);
-	proxy->priv->iter_rows = g_hash_table_new (NULL, NULL);
+	proxy->priv->modif_rows = g_hash_table_new (NULL, NULL);
 	proxy->priv->stamp = g_random_int ();
 
 	proxy->priv->add_null_entry = FALSE;
+	proxy->priv->idle_add_event_source = 0;
+
+	proxy->priv->sample_first_row = 0;
+	proxy->priv->sample_last_row = 0;
+	proxy->priv->sample_size = 300;
+	proxy->priv->current_nb_rows = 0;
 }
 
+static void adjust_displayed_chunck (GnomeDbDataProxy *proxy);
+static gboolean idle_add_model_rows (GnomeDbDataProxy *proxy);
+
+
 static void model_data_refreshed_cb (GnomeDbDataModel *model, GnomeDbDataProxy *proxy);
 
 /**
@@ -400,7 +403,6 @@ gnome_db_data_proxy_new (GnomeDbDataModel *model)
 		gnome_db_data_model_refresh (model, NULL);
 
 	proxy->priv->model_nb_cols = gda_data_model_get_n_columns (GDA_DATA_MODEL (model));
-	proxy->priv->model_nb_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (model));
 
 	proxy->priv->info_data_set = gnome_db_data_model_get_new_data_set (model);
 	proxy->priv->cols_non_modif_attrs = g_new0 (guint, proxy->priv->model_nb_cols);
@@ -418,6 +420,8 @@ gnome_db_data_proxy_new (GnomeDbDataModel *model)
 	g_signal_connect (G_OBJECT (model), "data-refreshed",
 			  G_CALLBACK (model_data_refreshed_cb), proxy);
 
+	adjust_displayed_chunck (proxy);
+
         return obj;
 }
 
@@ -439,7 +443,12 @@ gnome_db_data_proxy_dispose (GObject *object)
 	proxy = GNOME_DB_DATA_PROXY (object);
 	if (proxy->priv) {
 		gnome_db_base_nullify_check (GNOME_DB_BASE (object));
-		
+
+		if (proxy->priv->idle_add_event_source) {
+			g_idle_remove_by_data (proxy);
+			proxy->priv->idle_add_event_source = 0;
+		}
+
 		if (proxy->priv->info_data_set) {
 			g_object_unref (proxy->priv->info_data_set);
 			proxy->priv->info_data_set = NULL;
@@ -457,10 +466,10 @@ gnome_db_data_proxy_dispose (GObject *object)
 			proxy->priv->cols_non_modif_attrs = NULL;
 		}
 		
-		if (proxy->priv->iter_rows) {
+		if (proxy->priv->modif_rows) {
 			gnome_db_data_proxy_reset_all (proxy);
-			g_hash_table_destroy (proxy->priv->iter_rows);
-			proxy->priv->iter_rows = NULL;
+			g_hash_table_destroy (proxy->priv->modif_rows);
+			proxy->priv->modif_rows = NULL;
 		}
 	}
 
@@ -511,17 +520,9 @@ gnome_db_data_proxy_set_property (GObject              *object,
 				gint offset;
 				GtkTreePath *path;
 				GtkTreeIter iter;
-				GSList *list;
 
 				offset = proxy->priv->add_null_entry ? -1 : 1;
 
-				/* go through all the references to GtkTreeRow and add offset */
-				list = proxy->priv->all_modifs;
-				while (list) {
-					ROW_MODIF (list->data)->iter_row += offset;
-					list = g_slist_next (list);
-				}
-
 				/* GtkTreeModel interface management */
 				if (offset == 1) {
 					/* add a row in the GtkTreeModel */
@@ -586,41 +587,35 @@ model_data_refreshed_cb (GnomeDbDataModel *model, GnomeDbDataProxy *proxy)
 		 */
 		gint i, old_nb_rows;
 		GtkTreePath *path;
-		GtkTreeIter iter;
-		old_nb_rows = proxy->priv->model_nb_rows;
+		old_nb_rows = gnome_db_data_proxy_get_n_rows (proxy);
 
-		gnome_db_data_proxy_reset_all (proxy);
 		proxy->priv->model_nb_cols = gda_data_model_get_n_columns (GDA_DATA_MODEL (model));
-		proxy->priv->model_nb_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (model));
-		proxy->priv->model_refresh_pending = FALSE;
 
-		/* emit the GtkTreeModel::"row_changed" signal for all the rows which already existed */
-		for (i=0; (i < old_nb_rows) && (i < proxy->priv->model_nb_rows); i++) {
-			path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, i), -1);
-			data_proxy_get_iter (GTK_TREE_MODEL (proxy), &iter, path);
-			gtk_tree_model_row_changed (GTK_TREE_MODEL (proxy), path, &iter);
-			gtk_tree_path_free (path);
+		/* Free memory for the modifications */
+		while (proxy->priv->all_modifs) {
+			gint model_row = ROW_MODIF (proxy->priv->all_modifs->data)->model_row;
+			row_modifs_free (ROW_MODIF (proxy->priv->all_modifs->data));
+			if (model_row >= 0)
+				g_hash_table_remove (proxy->priv->modif_rows, GINT_TO_POINTER (model_row));
+			proxy->priv->all_modifs = g_slist_delete_link (proxy->priv->all_modifs, proxy->priv->all_modifs);
 		}
+		if (proxy->priv->new_rows) {
+			int j = g_slist_length (proxy->priv->new_rows);
 
-		if (old_nb_rows < proxy->priv->model_nb_rows) {
-			/* emit the GtkTreeModel::"row_inserted" signal for all the new rows */
-			for (; i < proxy->priv->model_nb_rows; i++) {
-				path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, i), -1);
-				data_proxy_get_iter (GTK_TREE_MODEL (proxy), &iter, path);
-				gtk_tree_model_row_inserted (GTK_TREE_MODEL (proxy), path, &iter);
-				gtk_tree_path_free (path);
-			}
-			proxy->priv->stamp = g_random_int ();
-		}
-		else {
 			/* emit the GtkTreeModel::"row_deleted" signal for all the new rows 
 			 * (using the same row number!) */
-			path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, i), -1);
-			for (; i < old_nb_rows; i++) 
+			path = gtk_tree_path_new_from_indices (proxy->priv->current_nb_rows + (proxy->priv->add_null_entry ? 1 : 0), -1);
+			for (i = 0; i < j ; i++) 
 				gtk_tree_model_row_deleted (GTK_TREE_MODEL (proxy), path);
 			gtk_tree_path_free (path);
 			proxy->priv->stamp = g_random_int ();
+				
+			g_slist_free (proxy->priv->new_rows);
+			proxy->priv->new_rows = NULL;
 		}
+
+		proxy->priv->model_refresh_pending = FALSE;
+		adjust_displayed_chunck (proxy);
 	}
 }
 
@@ -639,7 +634,7 @@ gnome_db_data_proxy_get_n_rows (GnomeDbDataProxy *proxy)
 	g_return_val_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy), -1);
 	g_return_val_if_fail (proxy->priv, -1);
 
-	nbrows = proxy->priv->model_nb_rows;
+	nbrows = proxy->priv->current_nb_rows;
 	nbrows += g_slist_length (proxy->priv->new_rows);
 	if (proxy->priv->add_null_entry)
 		nbrows += 1;
@@ -804,7 +799,7 @@ gnome_db_data_proxy_set_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter,
 		if (!rm->to_be_deleted && !rm->modif_values && (rm->model_row >= 0)) {
 			/* remove that RowModif, it's useless */
 			/* g_print ("Removing RM\n"); */
-			g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row));
+			g_hash_table_remove (proxy->priv->modif_rows, GINT_TO_POINTER (rm->model_row));
 			proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
 			row_modifs_free (rm);
 		}
@@ -896,19 +891,19 @@ find_or_create_row_modif (GnomeDbDataProxy *proxy, GtkTreeIter *iter, gint col,
 {
 	RowModif *rm;
 	RowValue *rv = NULL;
-	gint row;
+	gint model_row;
+
+	model_row = iter_to_model_numbers (proxy, GPOINTER_TO_INT (iter->user_data));
+	rm = find_row_modif_for_iter (proxy, iter);
 
-	row = GPOINTER_TO_INT (iter->user_data);
-	rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
 	if (!rm) {
 		/* create a new RowModif */
+		g_assert (model_row >= 0);
 		rm = row_modifs_new (proxy, iter);
-		rm->model_row = iter_to_model_numbers (proxy, row);
-		rm->iter_row = row;
-		g_hash_table_insert (proxy->priv->iter_rows, GINT_TO_POINTER (row), rm);
+		rm->model_row = model_row;
+		if (model_row >= 0)
+			g_hash_table_insert (proxy->priv->modif_rows, GINT_TO_POINTER (model_row), rm);
 		proxy->priv->all_modifs = g_slist_prepend (proxy->priv->all_modifs, rm);
-		
-		/* g_print ("New RM\n"); */	
 	}
 	else {
 		/* there are already some modifications to the row, try to catch the RowValue if available */
@@ -1008,26 +1003,25 @@ gnome_db_data_proxy_delete (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 {
 	RowModif *rm;
 	gboolean do_signal = FALSE;
-	gint row;
+	gint model_row;
 
 	g_return_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy));
 	g_return_if_fail (proxy->priv);
 	g_return_if_fail (iter);
 	g_return_if_fail (iter->stamp == proxy->priv->stamp);
 
-	row = GPOINTER_TO_INT (iter->user_data);
-	rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
+	model_row = iter_to_model_numbers (proxy, GPOINTER_TO_INT (iter->user_data));
+
+	rm = find_row_modif_for_iter (proxy, iter);
 	if (rm) {
 		if (! rm->to_be_deleted) {
 			if (rm->model_row < 0) {
 				/* remove the row completely because it does not exist in the data model */
 				GtkTreePath *path;
 				
-				g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row));
 				proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
 				proxy->priv->new_rows = g_slist_remove (proxy->priv->new_rows, rm);
 				row_modifs_free (rm);
-				re_compute_row_numbers (proxy);
 
 				proxy->priv->stamp = g_random_int ();				
 				path = gtk_tree_path_new ();
@@ -1044,9 +1038,8 @@ gnome_db_data_proxy_delete (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 	else {
 		/* the row is an existing row in the data model, create a new RowModif */
 		rm = row_modifs_new (proxy, iter);
-		rm->model_row = iter_to_model_numbers (proxy, row);
-		rm->iter_row = row;
-		g_hash_table_insert (proxy->priv->iter_rows, GINT_TO_POINTER (row), rm);
+		rm->model_row = model_row;
+		g_hash_table_insert (proxy->priv->modif_rows, GINT_TO_POINTER (model_row), rm);
 		proxy->priv->all_modifs = g_slist_prepend (proxy->priv->all_modifs, rm);
 		rm->to_be_deleted = TRUE;
 		do_signal = TRUE;
@@ -1074,22 +1067,22 @@ gnome_db_data_proxy_undelete (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 {
 	RowModif *rm;
 	gboolean do_signal = FALSE;
-	gint row;
+	gint model_row;
 
 	g_return_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy));
 	g_return_if_fail (proxy->priv);
 	g_return_if_fail (iter);
 	g_return_if_fail (iter->stamp == proxy->priv->stamp);
 
-	row = GPOINTER_TO_INT (iter->user_data);
-	rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
+	model_row = iter_to_model_numbers (proxy, GPOINTER_TO_INT (iter->user_data));
+	rm = find_row_modif_for_iter (proxy, iter);
 	if (rm) {
 		rm->to_be_deleted = FALSE;
 		if (!rm->modif_values) {
 			/* get rid of that RowModif */
 			do_signal= TRUE;
 			
-			g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row));
+			g_hash_table_remove (proxy->priv->modif_rows, GINT_TO_POINTER (model_row));
 			proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
 			row_modifs_free (rm);
 		}
@@ -1134,6 +1127,14 @@ gnome_db_data_proxy_get_iter_from_values (GnomeDbDataProxy *proxy, GtkTreeIter *
 	g_return_val_if_fail (iter, FALSE);
 	g_return_val_if_fail (values, FALSE);
 
+	/* if there are still some rows waiting to be added in the idle loop, then force them to be added
+	 * first, otherwise we might not find what we are looking for!
+	 */
+	if (proxy->priv->idle_add_event_source) {
+		g_idle_remove_by_data (proxy);
+		while (idle_add_model_rows (proxy)) ;
+	}
+
 	if (! gtk_tree_model_get_iter_first (GTK_TREE_MODEL (proxy), &intern))
 		return FALSE;
 
@@ -1206,19 +1207,22 @@ gnome_db_data_proxy_append (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 	g_return_if_fail (proxy->priv);
 	g_return_if_fail (iter);
 
+	/* RowModif structure */
 	rm = row_modifs_new (proxy, NULL);
 	rm->model_row = -1;
 	rm->orig_values = NULL; /* there is no original value */
 	rm->orig_values_size = 0;
-	rm->iter_row = gnome_db_data_proxy_get_n_rows (proxy);
 
 	proxy->priv->all_modifs = g_slist_prepend (proxy->priv->all_modifs, rm);
-	proxy->priv->new_rows = g_slist_prepend (proxy->priv->new_rows, rm);
-	g_hash_table_insert (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row), rm);
+	proxy->priv->new_rows = g_slist_append (proxy->priv->new_rows, rm);
 
+	/* new iter value */
 	path = gtk_tree_path_new ();
-	gtk_tree_path_append_index (path, rm->iter_row);
+	gtk_tree_path_append_index (path, proxy->priv->current_nb_rows + (proxy->priv->add_null_entry ? 1 : 0) +
+				    g_slist_length (proxy->priv->new_rows) - 1);
 	data_proxy_get_iter (GTK_TREE_MODEL (proxy), iter, path);
+
+	/* signal row insertion */
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL (proxy), path, iter);
 	gtk_tree_path_free (path);
 
@@ -1273,13 +1277,12 @@ gnome_db_data_proxy_reset_all (GnomeDbDataProxy *proxy)
 	GtkTreePath *path;
 	GtkTreeIter iter;
 
-	/* new rows are first treated and removed (no memory de-allocation here) */
+	/* new rows are first treated and removed (no memory de-allocation here, though) */
 	if (proxy->priv->new_rows) {
 		while (proxy->priv->new_rows) {
 			path = gtk_tree_path_new ();
-			gtk_tree_path_append_index (path, ROW_MODIF (proxy->priv->new_rows->data)->iter_row);
+			gtk_tree_path_append_index (path, proxy->priv->current_nb_rows + (proxy->priv->add_null_entry ? 1 : 0));
 			proxy->priv->new_rows = g_slist_delete_link (proxy->priv->new_rows, proxy->priv->new_rows);
-			re_compute_row_numbers (proxy);
 
 			gtk_tree_model_row_deleted (GTK_TREE_MODEL (proxy), path);
 			gtk_tree_path_free (path);
@@ -1290,16 +1293,16 @@ gnome_db_data_proxy_reset_all (GnomeDbDataProxy *proxy)
 
 	/* all modified rows are then treated (including memory de-allocation for new rows) */
 	while (proxy->priv->all_modifs) {
-		gint iter_row = ROW_MODIF (proxy->priv->all_modifs->data)->iter_row;
 		gint model_row = ROW_MODIF (proxy->priv->all_modifs->data)->model_row;
 		
 		row_modifs_free (ROW_MODIF (proxy->priv->all_modifs->data));
-		g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (iter_row));
+		if (model_row >= 0)
+			g_hash_table_remove (proxy->priv->modif_rows, GINT_TO_POINTER (model_row));
 		proxy->priv->all_modifs = g_slist_delete_link (proxy->priv->all_modifs, proxy->priv->all_modifs);
 		
 		if (model_row >= 0) {
 			/* not a new row */
-			path = gtk_tree_path_new_from_indices (iter_row, -1);
+			path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, model_row), -1);
 			data_proxy_get_iter (GTK_TREE_MODEL (proxy), &iter, path);
 			gtk_tree_model_row_changed (GTK_TREE_MODEL (proxy), path, &iter);
 			gtk_tree_path_free (path);
@@ -1334,7 +1337,7 @@ gnome_db_data_proxy_reset_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter, gin
 		path = gtk_tree_path_new ();
 		gtk_tree_path_append_index (path, row);
 		
-		rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
+		rm = find_row_modif_for_iter (proxy, iter);
 		if (rm && rm->modif_values) {
 			/* there are some modifications to the row */
 			GSList *list;
@@ -1351,12 +1354,11 @@ gnome_db_data_proxy_reset_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter, gin
 				rm->modif_values = g_slist_remove (rm->modif_values, rv);
 				if (!rm->modif_values && !rm->to_be_deleted) {
 					/* remove this RowList as well */
-					g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row));
 					proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
-					if (rm->model_row < 0) {
+					if (rm->model_row < 0) 
 						proxy->priv->new_rows = g_slist_remove (proxy->priv->new_rows, rm);
-						re_compute_row_numbers (proxy);
-					}
+					else
+						g_hash_table_remove (proxy->priv->modif_rows, GINT_TO_POINTER (rm->model_row));
 					row_modifs_free (rm);
 
 					proxy->priv->stamp = g_random_int ();
@@ -1373,6 +1375,8 @@ gnome_db_data_proxy_reset_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter, gin
 		g_warning ("Unknown GnomeDbDataProxy column: %d", col);
 }
 
+static gboolean commit_row_modif (GnomeDbDataProxy *proxy, RowModif *rm, GError **error);
+
 /**
  * gnome_db_data_proxy_commit_all
  * @proxy: a #GnomeDbDataProxy object
@@ -1387,23 +1391,22 @@ gnome_db_data_proxy_reset_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter, gin
 gboolean
 gnome_db_data_proxy_commit_all (GnomeDbDataProxy *proxy, GError **error)
 {
-	gboolean allok;
-	GtkTreeIter iter;
+	gboolean allok = TRUE;
+	GSList *list;
 
 	g_return_val_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy), FALSE);
 	g_return_val_if_fail (proxy->priv, FALSE);
 
 	gda_data_model_freeze (GDA_DATA_MODEL (proxy->priv->model));
+	proxy->priv->internal_changes ++;
 
-	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (proxy), &iter)) {
-		if (!gnome_db_data_proxy_commit_row (proxy, &iter, error))
-			allok = FALSE;
-		
-		while (allok && gtk_tree_model_iter_next (GTK_TREE_MODEL (proxy), &iter)) 
-			if (!gnome_db_data_proxy_commit_row (proxy, &iter, error)) 
-				allok = FALSE;
+	list = proxy->priv->all_modifs;
+	while (list && allok) {
+		allok = commit_row_modif (proxy, ROW_MODIF (list->data), error);
+		list = g_slist_next (list);
 	}
 
+	proxy->priv->internal_changes --;
 	gda_data_model_thaw (GDA_DATA_MODEL (proxy->priv->model)); /* => might make a refresh */
 
 	return allok;
@@ -1422,10 +1425,6 @@ gnome_db_data_proxy_commit_all (GnomeDbDataProxy *proxy, GError **error)
 gboolean
 gnome_db_data_proxy_commit_row (GnomeDbDataProxy *proxy, GtkTreeIter *iter, GError **error)
 {
-	gboolean err = FALSE;
-	RowModif *rm;
-	gint row;
-	
 	g_return_val_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy), FALSE);
 	g_return_val_if_fail (proxy->priv, FALSE);
 	g_return_val_if_fail (iter, FALSE);
@@ -1436,116 +1435,126 @@ gnome_db_data_proxy_commit_row (GnomeDbDataProxy *proxy, GtkTreeIter *iter, GErr
 	gnome_db_base_dump (proxy, 5);
 #endif
 
-	row = GPOINTER_TO_INT (iter->user_data);
-	rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
-	if (rm) {
-		proxy->priv->internal_changes ++;
-		if (rm->to_be_deleted) {
-			/* delete the row */
-			const GdaRow *gdarow;
+	return commit_row_modif (proxy, find_row_modif_for_iter (proxy, iter), error);
+}
+
+/*
+ * Commits the modifications held in one single RowModif structure.
+ *
+ * Returns: TRUE if no error occured
+ */
+static gboolean
+commit_row_modif (GnomeDbDataProxy *proxy, RowModif *rm, GError **error)
+{
+	gboolean err = FALSE;
+
+	if (!rm)
+		return TRUE;
+
+	proxy->priv->internal_changes ++;
+	if (rm->to_be_deleted) {
+		/* delete the row */
+		const GdaRow *gdarow;
+		
+		g_assert (rm->model_row >= 0);
+		gdarow = gda_data_model_get_row (GDA_DATA_MODEL (proxy->priv->model), rm->model_row);
+		err = ! gda_data_model_remove_row (GDA_DATA_MODEL (proxy->priv->model), gdarow);
+	}
+	else {
+		if (rm->model_row >= 0) {
+			/* update the row */
+			GdaRow *gdarow;
+			GSList *list;
+			GList *values = NULL;
+			gint i;
+			gboolean newvalue_found;
+			GdaValue *newvalue;
+			gboolean *is_default;
 			
-			g_assert (rm->model_row >= 0);
-			gdarow = gda_data_model_get_row (GDA_DATA_MODEL (proxy->priv->model), rm->model_row);
-			err = ! gda_data_model_remove_row (GDA_DATA_MODEL (proxy->priv->model), gdarow);
+			g_assert (rm->modif_values);
+			g_assert (rm->orig_values);
+			is_default = g_new0 (gboolean, proxy->priv->model_nb_cols);
+			for (i=0; i < rm->orig_values_size; i++) {
+				newvalue_found = FALSE;
+				newvalue = NULL;
+				
+				list = rm->modif_values;
+				while (list && !newvalue_found) {
+					if (ROW_VALUE (list->data)->model_column == i) {
+						newvalue = ROW_VALUE (list->data)->value;
+						newvalue_found = TRUE;
+						is_default [i] = ROW_VALUE (list->data)->attributes & 
+							GNOME_DB_VALUE_IS_DEFAULT;
+					}
+					list = g_slist_next (list);
+				}
+				if (!newvalue_found)
+					newvalue = rm->orig_values[i];
+				values = g_list_append (values, newvalue);
+			}
+			gdarow = gda_row_new_from_list (GDA_DATA_MODEL (proxy->priv->model), values);
+			g_list_free (values);
+			gda_row_set_number (gdarow, rm->model_row);
+			for (i = 0; i < proxy->priv->model_nb_cols; i++) {
+				if (is_default [i])
+					gda_row_set_is_default (gdarow, i, TRUE);
+			}
+			err = ! gda_data_model_update_row (GDA_DATA_MODEL (proxy->priv->model), gdarow);
+			gda_row_free (gdarow);
+			g_free (is_default);
 		}
 		else {
-			if (rm->model_row >= 0) {
-				/* update the row */
-				GdaRow *gdarow;
-				GSList *list;
-				GList *values = NULL;
-				gint i;
-				gboolean newvalue_found;
-				GdaValue *newvalue;
-				gboolean *is_default;
+			/* insert a new row */
+			GdaRow *gdarow;
+			GSList *list;
+			GList *values = NULL;
+			gint i;
+			GdaValue *newvalue;
+			gboolean *is_default;
+			
+			g_assert (rm->modif_values);
+			is_default = g_new0 (gboolean, proxy->priv->model_nb_cols);
+			for (i = 0; i < proxy->priv->model_nb_cols; i++) {
+				newvalue = NULL;
 				
-				g_assert (rm->modif_values);
-				g_assert (rm->orig_values);
-				is_default = g_new0 (gboolean, proxy->priv->model_nb_cols);
-				for (i=0; i < rm->orig_values_size; i++) {
-					newvalue_found = FALSE;
-					newvalue = NULL;
-					
-					list = rm->modif_values;
-					while (list && !newvalue_found) {
-						if (ROW_VALUE (list->data)->model_column == i) {
-							newvalue = ROW_VALUE (list->data)->value;
-							newvalue_found = TRUE;
-							is_default [i] = ROW_VALUE (list->data)->attributes & 
-								GNOME_DB_VALUE_IS_DEFAULT;
-						}
-						list = g_slist_next (list);
+				list = rm->modif_values;
+				while (list && !newvalue) {
+					if (ROW_VALUE (list->data)->model_column == i) {
+						newvalue = ROW_VALUE (list->data)->value;
+						is_default [i] = ROW_VALUE (list->data)->attributes & 
+							GNOME_DB_VALUE_IS_DEFAULT;
 					}
-					if (!newvalue_found)
-						newvalue = rm->orig_values[i];
-					values = g_list_append (values, newvalue);
+					list = g_slist_next (list);
 				}
-				gdarow = gda_row_new_from_list (GDA_DATA_MODEL (proxy->priv->model), values);
-				g_list_free (values);
-				gda_row_set_number (gdarow, rm->model_row);
-				for (i = 0; i < proxy->priv->model_nb_cols; i++) {
-					if (is_default [i])
-						gda_row_set_is_default (gdarow, i, TRUE);
-				}
-				err = ! gda_data_model_update_row (GDA_DATA_MODEL (proxy->priv->model), gdarow);
-				gda_row_free (gdarow);
-				g_free (is_default);
+				values = g_list_append (values, newvalue);
 			}
-			else {
-				/* insert a new row */
-				GdaRow *gdarow;
-				GSList *list;
-				GList *values = NULL;
-				gint i;
-				GdaValue *newvalue;
-				gboolean *is_default;
-				
-				g_assert (rm->modif_values);
-				is_default = g_new0 (gboolean, proxy->priv->model_nb_cols);
-				for (i = 0; i < proxy->priv->model_nb_cols; i++) {
-					newvalue = NULL;
-					
-					list = rm->modif_values;
-					while (list && !newvalue) {
-						if (ROW_VALUE (list->data)->model_column == i) {
-							newvalue = ROW_VALUE (list->data)->value;
-							is_default [i] = ROW_VALUE (list->data)->attributes & 
-								GNOME_DB_VALUE_IS_DEFAULT;
-						}
-						list = g_slist_next (list);
-					}
-					values = g_list_append (values, newvalue);
+			
+			gdarow = gda_row_new_from_list (GDA_DATA_MODEL (proxy->priv->model), values);
+			g_list_free (values);
+			for (i = 0; i < proxy->priv->model_nb_cols; i++) {
+				if (is_default [i]) {
+					gda_row_set_is_default (gdarow, i, TRUE);
 				}
-				
-				gdarow = gda_row_new_from_list (GDA_DATA_MODEL (proxy->priv->model), values);
-				g_list_free (values);
-				for (i = 0; i < proxy->priv->model_nb_cols; i++) {
-					if (is_default [i]) {
-						gda_row_set_is_default (gdarow, i, TRUE);
-					}
-				}
-				err = ! gda_data_model_append_row (GDA_DATA_MODEL (proxy->priv->model), gdarow);
-				gda_row_free (gdarow);
-				g_free (is_default);
 			}
-		}
-		proxy->priv->internal_changes --;
-
-		if (!err) {
-			/* data refresh */
-			if (proxy->priv->model_refresh_pending)
-				model_data_refreshed_cb (proxy->priv->model, proxy);
+			err = ! gda_data_model_append_row (GDA_DATA_MODEL (proxy->priv->model), gdarow);
+			gda_row_free (gdarow);
+			g_free (is_default);
 		}
 	}
-
-	if (err) 
+	proxy->priv->internal_changes --;
+	
+	if (!err) {
+		/* data refresh */
+		if (proxy->priv->model_refresh_pending)
+			model_data_refreshed_cb (proxy->priv->model, proxy);
+	}
+	else
 		g_set_error (error, GNOME_DB_DATA_PROXY_ERROR, GNOME_DB_DATA_PROXY_COMMIT_ERROR,
 			     _("Can't commit the modified values"));
 
 	return !err;
 }
 
-
 /*
  * GtkTreeModel interface
  *
@@ -1662,7 +1671,7 @@ data_proxy_get_path (GtkTreeModel *tree_model, GtkTreeIter *iter)
 static void
 data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column, GValue *value)
 {
-	gint row;
+	gint model_row;
 	GnomeDbDataProxy *proxy;
 	RowModif *rm;
 	gboolean value_set = FALSE;
@@ -1676,7 +1685,7 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 
 	g_value_init (value, data_proxy_get_column_type (tree_model, column));
 
-	row = GPOINTER_TO_INT (iter->user_data);
+	model_row = iter_to_model_numbers (proxy, GPOINTER_TO_INT (iter->user_data));
 
 	if (column < 0) {
 		switch (column) {
@@ -1685,7 +1694,7 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 			value_set = TRUE;
 			break;
 		case PROXY_COL_MODEL_ROW:
-			g_value_set_int (value, iter_to_model_numbers (proxy, row));
+			g_value_set_int (value, model_row);
 			value_set = TRUE;
 			break;
 		case PROXY_COL_MODEL_POINTER:
@@ -1693,12 +1702,12 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 			value_set = TRUE;
 			break;
 		case PROXY_COL_MODIFIED:
-			rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
+			rm = find_row_modif_for_iter (proxy, iter);
 			g_value_set_boolean (value, rm && rm->modif_values ? TRUE : FALSE);
 			value_set = TRUE;
 			break;
 		case PROXY_COL_TO_DELETE:
-			rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
+			rm = find_row_modif_for_iter (proxy, iter);
 			g_value_set_boolean (value, rm && rm->to_be_deleted ? TRUE : FALSE);
 			value_set = TRUE;
 			break;
@@ -1713,7 +1722,7 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 									    1 <=> attributes
 									    2 <=> original value */
 
-			rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
+			rm = find_row_modif_for_iter (proxy, iter);
 			if (what == 2) { /* original value */
 				if (rm) {
 					if (rm->orig_values)
@@ -1723,7 +1732,6 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 						g_value_set_pointer (value, NULL);
 				}
 				else {
-					gint model_row = iter_to_model_numbers (proxy, row);
 					g_value_set_pointer (value,
 							     gda_data_model_get_value_at (GDA_DATA_MODEL (proxy->priv->model), 
 											  model_col, model_row));
@@ -1754,8 +1762,6 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 				}
 
 				if (!value_has_modifs) {
-					gint model_row = iter_to_model_numbers (proxy, row);
-					
 					value_set = TRUE;
 					if (model_row >= 0) {
 						/* existing row */
@@ -1984,3 +1990,230 @@ gnome_db_data_proxy_dump (GnomeDbDataProxy *proxy, guint offset)
 	g_free (offstr);
 }
 #endif
+
+/**
+ * gnome_db_data_proxy_set_sample_size
+ * @proxy: a #GnomeDbDataProxy object
+ * @sample_size: the requested size of a chunck, or 0
+ *
+ * Sets the size of each chunck of fata to display: the maximum number of rows which
+ * can be displayed at a time. The default value is arbitrary 300 as it is big enough to
+ * be able to display quite a lot of data, but small enough to avoid too much data
+ * displayed at the same time.
+ *
+ * Note: the rows which have been added but not yet commited will always be displayed
+ * regardless of the current chunck of data, and the modified rows which are not visible
+ * when the displayed chunck of data changes are still held as modified rows.
+ *
+ * To remove the chuncking of the data to display, simply pass @sample_size the 0 value.
+ */
+void
+gnome_db_data_proxy_set_sample_size (GnomeDbDataProxy *proxy, gint sample_size)
+{
+	gint new_sample_size;
+	g_return_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy));
+	g_return_if_fail (proxy->priv);
+
+	new_sample_size = sample_size <= 0 ? 0 : sample_size;
+	if (proxy->priv->sample_size != new_sample_size) {
+		proxy->priv->sample_size = new_sample_size;
+		adjust_displayed_chunck (proxy);
+	}
+}
+
+/**
+ * gnome_db_data_proxy_get_sample_size
+ * @proxy: a #GnomeDbDataProxy object
+ *
+ * Get the size of each chunk of data displayed at a time.
+ *
+ * Returns: the chunck (or sample) size, or 0 if chunking is disabled.
+ */
+gint
+gnome_db_data_proxy_get_sample_size (GnomeDbDataProxy *proxy)
+{
+	g_return_val_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy), 0);
+	g_return_val_if_fail (proxy->priv, 0);
+
+	return proxy->priv->sample_size;
+}
+
+/**
+ * gnome_db_data_proxy_set_sample_start
+ * @proxy: a #GnomeDbDataProxy object
+ * @sample_start: the number of the first row to be displayed
+ *
+ * Sets the number of the first row to be displayed.
+ */
+void
+gnome_db_data_proxy_set_sample_start (GnomeDbDataProxy *proxy, gint sample_start)
+{
+	g_return_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy));
+	g_return_if_fail (proxy->priv);
+	g_return_if_fail (sample_start >= 0);
+
+	if (proxy->priv->sample_first_row != sample_start) {
+		proxy->priv->sample_first_row = sample_start;
+		adjust_displayed_chunck (proxy);
+	}
+}
+
+/**
+ * gnome_db_data_proxy_get_sample_start
+ * @proxy: a #GnomeDbDataProxy object
+ *
+ * Get the row number of the first row to be displayed.
+ *
+ * Returns: the number of the first row being displayed.
+ */
+gint
+gnome_db_data_proxy_get_sample_start (GnomeDbDataProxy *proxy)
+{
+	g_return_val_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy), 0);
+	g_return_val_if_fail (proxy->priv, 0);
+
+	return proxy->priv->sample_first_row;
+}
+
+/**
+ * gnome_db_data_proxy_get_sample_end
+ * @proxy: a #GnomeDbDataProxy object
+ *
+ * Get the row number of the last row to be displayed.
+ *
+ * Returns: the number of the last row being displayed.
+ */
+gint
+gnome_db_data_proxy_get_sample_end (GnomeDbDataProxy *proxy)
+{
+	g_return_val_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy), 0);
+	g_return_val_if_fail (proxy->priv, 0);
+
+	return proxy->priv->sample_last_row;	
+}
+
+/*
+ * Adjusts the values of the first and last rows to be displayed depending
+ * on the sample size, and update the GtkTreeModel interface
+ */
+static void
+adjust_displayed_chunck (GnomeDbDataProxy *proxy)
+{
+	gint i, old_nb_rows, new_nb_rows;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	gint model_nb_rows;
+
+	g_return_if_fail (proxy->priv->model);
+
+	/*
+	 * Stop idle adding of rows if necessary
+	 */
+	if (proxy->priv->idle_add_event_source) {
+		g_idle_remove_by_data (proxy);
+		proxy->priv->idle_add_event_source = 0;
+	}
+
+	/*
+	 * Compute chuncks limits
+	 */
+	old_nb_rows = proxy->priv->current_nb_rows;
+	model_nb_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (proxy->priv->model));
+	if (proxy->priv->sample_size >= 0) {
+		if (proxy->priv->sample_first_row >= model_nb_rows) 
+			proxy->priv->sample_first_row = proxy->priv->sample_size * ((model_nb_rows - 1) / proxy->priv->sample_size);
+
+		proxy->priv->sample_last_row = proxy->priv->sample_first_row + proxy->priv->sample_size - 1;
+		if (proxy->priv->sample_last_row >= model_nb_rows)
+			proxy->priv->sample_last_row = model_nb_rows - 1;
+		new_nb_rows = proxy->priv->sample_last_row - proxy->priv->sample_first_row + 1;
+	}
+	else {
+		proxy->priv->sample_first_row = 0;
+		proxy->priv->sample_last_row = model_nb_rows - 1;
+		new_nb_rows = proxy->priv->sample_last_row;
+	}
+
+	/*
+	 * GtkTreeModel: update all the rows which can be updated:
+	 * emit the GtkTreeModel::"row_changed" signal for all the rows which already existed
+	 */
+	for (i=0; (i < old_nb_rows) && (i < new_nb_rows); i++) {
+		path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, proxy->priv->sample_first_row + i), -1);
+		data_proxy_get_iter (GTK_TREE_MODEL (proxy), &iter, path);
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (proxy), path, &iter);
+		gtk_tree_path_free (path);
+	}
+	
+	if (old_nb_rows < new_nb_rows) {
+		/*
+		 * GtkTreeModel: insert the missing rows in the idle loop
+		 */
+		proxy->priv->stamp = g_random_int ();
+		proxy->priv->idle_add_event_source = g_idle_add ((GSourceFunc) idle_add_model_rows, proxy);
+
+		proxy->priv->current_nb_rows = old_nb_rows;
+	}
+	else {
+		/*
+		 * GtkTreeModel: delete all the remaining rows:
+		 * emit the GtkTreeModel::"row_deleted" signal for all the new rows 
+		 * (using the same row number!) 
+		 */
+		path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, proxy->priv->sample_first_row + i), -1);
+		for (; i < old_nb_rows; i++) 
+			gtk_tree_model_row_deleted (GTK_TREE_MODEL (proxy), path);
+		gtk_tree_path_free (path);
+		proxy->priv->stamp = g_random_int ();
+
+		proxy->priv->current_nb_rows = new_nb_rows;
+	}
+}
+
+/*
+ * Called in an idle loop to "add" rows to the proxy. The only effect is to
+ * "declare" more rows in the GtkTreeModel interface. Returns FALSE when
+ * it does not need to be called anymore.
+ *
+ * Rem: must be called only by the adjust_displayed_chunck() function.
+ */
+static gboolean
+idle_add_model_rows (GnomeDbDataProxy *proxy)
+{
+	gint tmp_current_nb_rows;
+	gint model_nb_rows;
+	gint i = 0;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+
+#define IDLE_STEP 50
+
+	g_return_val_if_fail (proxy->priv->model, FALSE);
+
+	model_nb_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (proxy->priv->model));
+	if (proxy->priv->sample_size > 0) {
+		tmp_current_nb_rows = proxy->priv->sample_size;
+		if (tmp_current_nb_rows >= model_nb_rows)
+			tmp_current_nb_rows = model_nb_rows;
+	}
+	else
+		tmp_current_nb_rows = model_nb_rows;
+	
+	while ((i < IDLE_STEP) && (proxy->priv->current_nb_rows < tmp_current_nb_rows)) {
+		proxy->priv->current_nb_rows ++;
+		path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, 
+									      proxy->priv->sample_first_row + proxy->priv->current_nb_rows - 1), -1);
+		data_proxy_get_iter (GTK_TREE_MODEL (proxy), &iter, path);
+		gtk_tree_model_row_inserted (GTK_TREE_MODEL (proxy), path, &iter);
+		gtk_tree_path_free (path);
+		i++;
+	}
+	proxy->priv->stamp = g_random_int ();
+
+	if (i < IDLE_STEP) {
+		proxy->priv->idle_add_event_source = 0;
+		return FALSE;
+	}
+	else
+		return TRUE;
+}
