@@ -79,7 +79,7 @@ struct _GnomeDbDataProxyClass
 	GnomeDbBaseClass                    class;
 };
 
-guint             gnome_db_data_proxy_get_type             (void);
+GType             gnome_db_data_proxy_get_type             (void);
 GObject          *gnome_db_data_proxy_new                  (GnomeDbDataModel *model);
 
 gboolean          gnome_db_data_proxy_is_read_only         (GnomeDbDataProxy *proxy);
