@@ -30,6 +30,8 @@
 #include "gnome-db-entity.h"
 #include "gnome-db-server-data-type.h"
 #include "gnome-db-qfield.h"
+#include "gnome-db-server.h"
+#include "gnome-db-model-wrapper.h"
 
 /* 
  * Main static functions 
@@ -98,7 +100,7 @@ GQuark gnome_db_data_set_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_data_set_get_type (void)
 {
 	static GType type = 0;
@@ -338,6 +340,351 @@ gnome_db_data_set_new_copy (GnomeDbDataSet *orig, GHashTable *replacements)
 	return obj;
 }
 
+/**
+ * gnome_db_data_set_new_from_spec
+ * @xml_spec: a string
+ * @error: a place to store the error, or %NULL
+ *
+ * Creates a new #GnomeDbDataSet object from the @xml_spec
+ * specifications
+ *
+ * Returns: a new object, or %NULL if an error occured
+ */
+GObject *
+gnome_db_data_set_new_from_spec (GnomeDbDict *dict, const gchar *xml_spec, GError **error)
+{
+	GObject *obj = NULL;
+	GSList *params = NULL, *sources = NULL;
+	GSList *list;
+	
+	xmlDocPtr doc;
+	xmlNodePtr root, cur;
+	GnomeDbServer *srv;
+	gboolean allok = TRUE;
+	gchar *str;
+
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
+	srv = gnome_db_dict_get_server (ASSERT_DICT (dict));
+
+	/* string parsing */
+	doc = xmlParseMemory (xml_spec, strlen (xml_spec));
+	if (!doc)
+		return NULL;
+	root = xmlDocGetRootElement (doc);
+	if (strcmp (root->name, "data-set-spec") != 0){
+		g_set_error (error, GNOME_DB_DATA_SET_ERROR, GNOME_DB_DATA_SET_XML_SPEC_ERROR,
+			     _("Spec's root node != 'data-set-spec': '%s'"), root->name);
+		return NULL;
+	}
+
+	/* Making the list of parameters */
+	root = root->xmlChildrenNode;
+	while (xmlNodeIsText (root)) 
+		root = root->next; 
+
+	if (strcmp (root->name, "parameters") != 0){
+		g_set_error (error, GNOME_DB_DATA_SET_ERROR, GNOME_DB_DATA_SET_XML_SPEC_ERROR,
+			     _("Missing node <parameters>: '%s'"), root->name);
+		return NULL;
+	}
+
+	for (cur = root->xmlChildrenNode; (cur != NULL) && allok; cur = cur->next) {
+		if (xmlNodeIsText (cur)) 
+			continue;
+
+		if (!strcmp (cur->name, "parameter")) {
+			GnomeDbParameter *param = NULL;
+			GnomeDbServerDataType *dtype = NULL;
+			gchar *str;
+			gboolean dtype_created = FALSE;
+			
+			/* find data type and create GnomeDbParameter */
+			str = xmlGetProp(cur, "dbmstype");
+			if (str) {
+				dtype = gnome_db_server_get_data_type_by_name (srv, str);
+				g_free (str);
+			}
+			if (!dtype) {
+				str = xmlGetProp (cur, "gdatype");
+				if (str) {
+					GdaValueType gtype;
+
+					gtype = gda_type_from_string (str);
+					if (gtype != GDA_VALUE_TYPE_UNKNOWN) {
+						GnomeDbServerInfo *info;
+						gint i;
+						
+						info = gnome_db_server_get_server_info (srv);
+						if (info) {
+							for (i = 0; (i < info->value_nb_tests_types) && (!dtype); i ++) {
+								if (info->value_test_gda_types [i] == gtype)
+									dtype = gnome_db_server_get_data_type_by_name (srv, 
+											        info->value_test_data_types [i]);
+							}
+						}
+
+						if (!dtype) {
+							/* create a GnomeDbServerDataType for that 'gda-type' */
+							dtype = GNOME_DB_SERVER_DATA_TYPE (gnome_db_server_data_type_new (srv));
+							gnome_db_server_data_type_set_sqlname (dtype, str);
+							gnome_db_server_data_type_set_gda_type (dtype, gtype);
+							gnome_db_server_declare_custom_data_type (srv, dtype);
+							dtype_created = TRUE;
+						}
+					}
+					g_free (str);					
+				}
+			}
+
+			if (!dtype) {
+				str = xmlGetProp(cur, "name");
+				
+				g_set_error (error, GNOME_DB_DATA_SET_ERROR, GNOME_DB_DATA_SET_XML_SPEC_ERROR,
+					     _("Can't find a data type for parameter '%s'"), str ? str : "unnamed");
+				g_free (str);
+
+				allok = FALSE;
+				continue;
+			}
+
+			param = GNOME_DB_PARAMETER (gnome_db_parameter_new (dict, dtype));
+			if (dtype_created)
+				g_object_unref (dtype);
+			params = g_slist_append (params, param);
+
+			/* set parameter's attributes */
+			str = xmlGetProp(cur, "id");
+			if (str) {
+				g_object_set (G_OBJECT (param), "string_id", str, NULL);
+				g_free (str);
+			}
+			str = xmlGetProp(cur, "name");
+			if (str) {
+				gnome_db_base_set_name (GNOME_DB_BASE (param), str);
+				g_free (str);
+			}
+			str = xmlGetProp(cur, "descr");
+			if (str) {
+				gnome_db_base_set_description (GNOME_DB_BASE (param), str);
+				g_free (str);
+			}
+			str = xmlGetProp(cur, "null-ok");
+			if (str) {
+				gnome_db_parameter_set_not_null (param, (*str == 'T') || (*str == 't') ? FALSE : TRUE);
+				g_free (str);
+			}
+			else
+				gnome_db_parameter_set_not_null (param, FALSE);
+			str = xmlGetProp(cur, "plugin");
+			if (str) {
+				g_object_set (G_OBJECT (param), "handler_plugin", str, NULL);
+				g_free (str);
+			}
+
+			str = xmlGetProp(cur, "source");
+			if (str) 
+				g_object_set_data (G_OBJECT (param), "source", str);
+		}
+	}
+
+	/* Parameters' sources, not mandatory */
+	root = root->next;
+	while (xmlNodeIsText (root)) 
+		root = root->next; 
+	if (allok && root && !strcmp (root->name, "sources")){
+		for (cur = root->xmlChildrenNode; (cur != NULL) && allok; cur = cur->next) {
+			if (xmlNodeIsText (cur)) 
+				continue;
+
+			if (!strcmp (cur->name, "data-array")) {
+				GdaDataModel *model;
+
+				str = xmlGetProp(cur, "name");
+				model = gda_data_model_array_new_from_xml_node (cur, error);
+				if (model) {
+					GnomeDbModelWrapper *wrapper;
+
+					wrapper = gnome_db_model_wrapper_new (dict, model);
+					if (str)
+						gnome_db_base_set_name (GNOME_DB_BASE (wrapper), str);
+					g_object_unref (model);
+					sources = g_slist_prepend (sources, wrapper);
+				}
+				else 
+					allok = FALSE; /* error has already been set */
+				g_free (str);
+			}
+		}
+	}
+	
+	/* affecting parameters' sources fields to each parameter */
+	list = params;
+	while (list && allok) {
+		str = g_object_get_data (G_OBJECT (list->data), "source");
+		if (str) {
+			gchar *ptr1, *ptr2=NULL, *tok;
+			
+			ptr1 = strtok_r (str, ":", &tok);
+			if (ptr1)
+				ptr2 = strtok_r (NULL, ":", &tok);
+			
+			if (ptr1 && ptr2) {
+				GSList *tmp = sources;
+				GnomeDbModelWrapper *wrapper = NULL;
+				while (tmp && !wrapper) {
+					if (!strcmp (gnome_db_base_get_name (GNOME_DB_BASE (tmp->data)), ptr1))
+						wrapper = GNOME_DB_MODEL_WRAPPER (tmp->data);
+					tmp = g_slist_next (tmp);
+				}
+
+				if (wrapper) {
+					gint fno;
+
+					fno = atoi (ptr2);
+					if ((fno <= 0) ||
+					    (fno > gda_data_model_get_n_columns (GDA_DATA_MODEL (wrapper)))) {
+						g_set_error (error, GNOME_DB_DATA_SET_ERROR, GNOME_DB_DATA_SET_XML_SPEC_ERROR,
+							     _("Field number %d not found in source named '%s'"), fno, ptr1); 
+						allok = FALSE;	
+					}
+					else {
+						GnomeDbField *field;
+
+						field = gnome_db_entity_get_field_by_index (GNOME_DB_ENTITY (wrapper), fno - 1);
+						if (!gnome_db_parameter_set_source_field (GNOME_DB_PARAMETER (list->data),
+											  field, error))
+							allok = FALSE;
+						else {
+							/* rename the wrapper with the current param's name */
+							g_object_set_data (G_OBJECT (wrapper), "newname", 
+									   gnome_db_base_get_name (GNOME_DB_BASE (list->data)));
+							g_object_set_data (G_OBJECT (wrapper), "newdescr", 
+									   gnome_db_base_get_description (GNOME_DB_BASE (list->data)));
+						}
+					}
+				}
+				else {
+					g_set_error (error, GNOME_DB_DATA_SET_ERROR, GNOME_DB_DATA_SET_XML_SPEC_ERROR,
+						     _("Can't find parameter source named '%s'"), ptr1); 
+					allok = FALSE;
+				}
+			}
+			else {
+				g_set_error (error, GNOME_DB_DATA_SET_ERROR, GNOME_DB_DATA_SET_XML_SPEC_ERROR,
+					     _("Parameter source specification should be \"<source name>:<field pos>\": '%s'"), 
+					     str);
+				allok = FALSE;
+			}
+
+			g_object_set_data (G_OBJECT (list->data), "source", NULL);
+			g_free (str);
+		}
+		list = g_slist_next (list);
+	}
+
+	/* setting prepared new names from sources (wrappers) */
+	list = sources;
+	while (list) {
+		str = g_object_get_data (G_OBJECT (list->data), "newname");
+		if (str) {
+			gnome_db_base_set_name (GNOME_DB_BASE (list->data), str);
+			g_object_set_data (G_OBJECT (list->data), "newname", NULL);
+		}
+		str = g_object_get_data (G_OBJECT (list->data), "newdescr");
+		if (str) {
+			gnome_db_base_set_description (GNOME_DB_BASE (list->data), str);
+			g_object_set_data (G_OBJECT (list->data), "newdescr", NULL);
+		}
+		list = g_slist_next (list);
+	}
+
+	/* parameters' values, constraints: TODO */
+	
+	/* GnomeDbDataSet creation */
+	if (allok) 
+		obj = gnome_db_data_set_new (dict, params);
+
+	g_slist_foreach (params, (GFunc) g_object_unref, NULL);
+	g_slist_free (params);
+	g_slist_foreach (sources, (GFunc) g_object_unref, NULL);
+	g_slist_free (sources);
+
+	xmlFreeDoc(doc);
+	return obj;
+}
+
+/**
+ * gnome_db_data_set_get_spec
+ * @dataset: a #GnomeDbDataSet object
+ *
+ * Get the specification as an XML string. See the gnome_db_data_set_new_from_spec()
+ * form more information about the XML specification string format.
+ *
+ * Returns: a new string
+ */
+gchar *
+gnome_db_data_set_get_spec (GnomeDbDataSet *dataset)
+{
+	xmlDocPtr doc;
+	xmlNodePtr root;
+	xmlChar *xmlbuff;
+	int buffersize;
+	GSList *list;
+
+	g_return_val_if_fail (dataset && IS_GNOME_DB_DATA_SET (dataset), NULL);
+
+	doc = xmlNewDoc ("1.0");
+	g_return_val_if_fail (doc, NULL);
+	root = xmlNewDocNode (doc, NULL, "data-set-spec", NULL);
+	xmlDocSetRootElement (doc, root);
+
+	/* parameters list */
+	list = dataset->parameters;
+	while (list) {
+		xmlNodePtr node;
+		GnomeDbParameter *param = GNOME_DB_PARAMETER (list->data);
+		GnomeDbServerDataType *dtype;
+		gchar *str;
+
+		node = xmlNewTextChild (root, NULL, "parameter", NULL);
+		g_object_get (G_OBJECT (param), "string_id", &str, NULL);
+		if (str) {
+			xmlSetProp (node, "id", str);
+			g_free (str);
+		}
+		str = gnome_db_base_get_name (GNOME_DB_BASE (param));
+		if (str)
+			xmlSetProp (node, "name", str);		
+		str = gnome_db_base_get_description (GNOME_DB_BASE (param));
+		if (str)
+			xmlSetProp (node, "descr", str);		
+
+		dtype = gnome_db_parameter_get_data_type (param);
+		if (dtype) {
+			xmlSetProp (node, "dbmstype", gnome_db_server_data_type_get_sqlname (dtype));
+			xmlSetProp (node, "gdatype", gda_type_to_string (gnome_db_server_data_type_get_gda_type (dtype)));
+		}
+
+		xmlSetProp (node, "null-ok", gnome_db_parameter_get_not_null (param) ? "FALSE" : "TRUE");
+		g_object_get (G_OBJECT (param), "handler_plugin", &str, NULL);
+		if (str) {
+			xmlSetProp (node, "plugin", str);
+			g_free (str);
+		}
+		
+		list = g_slist_next (list);
+	}
+
+	/* parameters' values, sources, constraints: TODO */
+
+	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
+	g_print ((gchar *) xmlbuff);
+	
+	xmlFreeDoc(doc);
+	return (gchar *) xmlbuff;
+}
+
+
 static void
 nullified_param_cb (GnomeDbParameter *param, GnomeDbDataSet *dataset)
 {
@@ -524,7 +871,8 @@ compute_dataset_nodes (GnomeDbDataSet *dataset)
 				 * call to this function */
 				g_warning ("The GnomeDbDataSet::%s() function does not yet know how to compute\n"
 					   "new GnomeDbDataSetNode structures when adding parameters which have a source field:\n"
-					   "The layout of the new GnomeDbDataSetNode list may not be the correct one!", __FUNCTION__);
+					   "The layout of the new GnomeDbDataSetNode list may not be the correct one!", 
+					   __FUNCTION__);
 
 			ent = gnome_db_field_get_entity (ref_field);
 
@@ -547,6 +895,8 @@ compute_dataset_nodes (GnomeDbDataSet *dataset)
 												   NULL));
 						gnome_db_base_set_name (GNOME_DB_BASE (rs),
 									gnome_db_base_get_name (GNOME_DB_BASE (list->data)));
+						gnome_db_base_set_description (GNOME_DB_BASE (rs),
+									gnome_db_base_get_description (GNOME_DB_BASE (list->data)));
 						g_hash_table_insert (new_rs, ent, rs);
 						g_object_get (G_OBJECT (rs), "init_query_replacements", &repl, NULL);
 					}
@@ -571,6 +921,8 @@ compute_dataset_nodes (GnomeDbDataSet *dataset)
 						g_hash_table_insert (hash, query, node);
 						g_hash_table_insert (node_repls, node, repl);
 					}
+					else
+						g_object_ref (G_OBJECT (ent));
 
 					dataset->nodes = g_slist_append (dataset->nodes, node);
 				}
@@ -948,9 +1300,11 @@ gnome_db_data_set_is_valid (GnomeDbDataSet *dataset)
 		if (!gnome_db_parameter_is_valid (GNOME_DB_PARAMETER (params->data))) 
 			retval = FALSE;
 		
-		/* g_print ("== PARAM %p: valid= %d, value=%s\n", params->data, gnome_db_parameter_is_valid (GNOME_DB_PARAMETER (params->data)), */
-/* 			 gnome_db_parameter_get_value (GNOME_DB_PARAMETER (params->data)) ?  */
-/* 			 gda_value_stringify (gnome_db_parameter_get_value (GNOME_DB_PARAMETER (params->data))) : "Null"); */
+#ifdef debug_NO
+		g_print ("== PARAM %p: valid= %d, value=%s\n", params->data, gnome_db_parameter_is_valid (GNOME_DB_PARAMETER (params->data)),
+			 gnome_db_parameter_get_value (GNOME_DB_PARAMETER (params->data)) ?
+			 gda_value_stringify (gnome_db_parameter_get_value (GNOME_DB_PARAMETER (params->data))) : "Null");
+#endif
 		params = g_slist_next (params);
 	}
 
@@ -993,6 +1347,39 @@ gnome_db_data_set_needs_user_input (GnomeDbDataSet *dataset)
 	return retval;
 }
 
+/**
+ * gnome_db_data_set_find_parameter
+ * @dataset: a #GnomeDbDataSet object
+ * @param_name: the name of the requested parameter
+ *
+ * Finds a #GnomeDbParameter using its name
+ *
+ * Returns: a #GnomeDbParameter or %NULL
+ */
+GnomeDbParameter *
+gnome_db_data_set_find_parameter (GnomeDbDataSet *dataset, const gchar *param_name)
+{
+	GnomeDbParameter *param = NULL;
+	GSList *list;
+	gchar *string_id;
+
+	g_return_val_if_fail (dataset && IS_GNOME_DB_DATA_SET (dataset), NULL);
+	g_return_val_if_fail (dataset->priv, NULL);
+
+	list = dataset->parameters;
+	while (list && !param) {
+		if (!strcmp (gnome_db_base_get_name (GNOME_DB_BASE (list->data)), param_name))
+			param = GNOME_DB_PARAMETER (list->data);
+		if (!param) {
+			g_object_get (G_OBJECT (list->data), "string_id", &string_id, NULL);
+			if (!strcmp (string_id, param_name))
+				param = GNOME_DB_PARAMETER (list->data);
+		}
+		list = g_slist_next (list);
+	}
+
+	return param;
+}
 
 /**
  * gnome_db_data_set_find_parameter_for_field
