@@ -82,7 +82,7 @@ struct _GnomeDbDictClass
 	void (*changed)          (GnomeDbDict * dict);
 };
 
-guint           gnome_db_dict_get_type            (void);
+GType           gnome_db_dict_get_type            (void);
 GObject        *gnome_db_dict_new                 (void);
 
 guint           gnome_db_dict_get_id_serial       (GnomeDbDict *dict);
