@@ -27,18 +27,26 @@
 #include <libgnomedb/gnome-db-dsn-config.h>
 #include <libgnomedb/gnome-db-provider-selector.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <libgnomedb/gnome-db-data-set.h>
+#include <libgnomedb/gnome-db-parameter.h>
+#include <libgnomedb/gnome-db-basic-form.h>
+#include <libgnomedb/gnome-db-server-data-type.h>
+#include <libgnomedb/gnome-db-data-handler.h>
 #include <glib/gi18n-lib.h>
+#include "gnome-db-dsn-spec.h"
 
 #define PARENT_TYPE GTK_TYPE_VBOX
 
 struct _GnomeDbDsnConfigPrivate {
 	GtkWidget *wname;
 	GtkWidget *wprovider;
-	GtkWidget *wstring;
 	GtkWidget *wdesc;
 	GtkWidget *wuser;
 	GtkWidget *wpassword;
 	GtkWidget *is_global;
+
+	GtkWidget *expander;
+	GtkWidget *dsn_spec;
 };
 
 static void gnome_db_dsn_config_class_init (GnomeDbDsnConfigClass *klass);
@@ -54,10 +62,6 @@ enum {
 static gint gnome_db_dsn_config_signals[LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class = NULL;
 
-/*
- * Private functions
- */
-
 /*
  * GnomeDbDsnConfig class implementation
  */
@@ -86,6 +90,12 @@ gnome_db_dsn_config_class_init (GnomeDbDsnConfigClass *klass)
 static void
 field_changed_cb (GtkWidget *widget, GnomeDbDsnConfig *config)
 {
+	if (widget == config->priv->wprovider) 
+		/* replace the expander's contents */
+		gnome_db_dsn_spec_set_provider (GNOME_DB_DSN_SPEC (config->priv->dsn_spec),
+						gnome_db_provider_selector_get_selected_provider 
+						(GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
+
 	g_signal_emit (config, gnome_db_dsn_config_signals[CHANGED], 0, NULL);
 }
 
@@ -100,6 +110,8 @@ gnome_db_dsn_config_init (GnomeDbDsnConfig *config, GnomeDbDsnConfigClass *klass
 {
 	GtkWidget *table;
 	GtkWidget *label;
+	GtkWidget *exp;
+	gchar *str;
 
 	g_return_if_fail (GNOME_DB_IS_DSN_CONFIG (config));
 
@@ -110,7 +122,12 @@ gnome_db_dsn_config_init (GnomeDbDsnConfig *config, GnomeDbDsnConfigClass *klass
 	table = gnome_db_new_table_widget (7, 3, FALSE);
 	gtk_box_pack_start (GTK_BOX (config), table, TRUE, TRUE, 0);
 
-	label = gnome_db_new_label_widget (_("Data source _name:"));
+	str = g_strdup_printf ("<b>%s</b>", _("Data source _name:"));
+	label = gtk_label_new ("");
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str);
+	g_free (str);
+	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wname = gnome_db_new_entry_widget (0, TRUE);
@@ -132,7 +149,12 @@ gnome_db_dsn_config_init (GnomeDbDsnConfig *config, GnomeDbDsnConfigClass *klass
 	gtk_table_attach (GTK_TABLE (table), config->priv->is_global, 1, 2, 1, 2,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 
-	label = gnome_db_new_label_widget (_("_Provider:"));
+	str = g_strdup_printf ("<b>%s</b>", _("_Provider:"));
+	label = gtk_label_new ("");
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str);
+	g_free (str);
+	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wprovider = gnome_db_provider_selector_new ();
@@ -143,50 +165,51 @@ gnome_db_dsn_config_init (GnomeDbDsnConfig *config, GnomeDbDsnConfigClass *klass
 	gtk_table_attach (GTK_TABLE (table), config->priv->wprovider, 1, 2, 2, 3,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 
-	label = gnome_db_new_label_widget (_("Connection _string:"));
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
-	config->priv->wstring = gnome_db_new_entry_widget (0, TRUE);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), config->priv->wstring);
-	gtk_widget_show (config->priv->wstring);
-	g_signal_connect (G_OBJECT (config->priv->wstring), "changed",
-			  G_CALLBACK (field_changed_cb), config);
-	gtk_table_attach (GTK_TABLE (table), config->priv->wstring, 1, 2, 3, 4,
-			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
-
 	label = gnome_db_new_label_widget (_("_Description:"));
 	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wdesc = gnome_db_new_entry_widget (0, TRUE);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), config->priv->wdesc);
 	gtk_widget_show (config->priv->wdesc);
 	g_signal_connect (G_OBJECT (config->priv->wdesc), "changed",
 			  G_CALLBACK (field_changed_cb), config);
-	gtk_table_attach (GTK_TABLE (table), config->priv->wdesc, 1, 2, 4, 5,
+	gtk_table_attach (GTK_TABLE (table), config->priv->wdesc, 1, 2, 3, 4,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 
 	label = gnome_db_new_label_widget (_("_Username:"));
 	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wuser = gnome_db_new_entry_widget (0, TRUE);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), config->priv->wuser);
 	gtk_widget_show (config->priv->wuser);
 	g_signal_connect (G_OBJECT (config->priv->wuser), "changed",
 			  G_CALLBACK (field_changed_cb), config);
-	gtk_table_attach (GTK_TABLE (table), config->priv->wuser, 1, 2, 5, 6,
+	gtk_table_attach (GTK_TABLE (table), config->priv->wuser, 1, 2, 4, 5,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 
 	label = gnome_db_new_label_widget (_("P_assword:"));
 	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 6, 7, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wpassword = gnome_db_new_entry_widget (0, TRUE);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), config->priv->wpassword);
 	gtk_entry_set_visibility (GTK_ENTRY (config->priv->wpassword), FALSE);
 	gtk_widget_show (config->priv->wpassword);
 	g_signal_connect (G_OBJECT (config->priv->wpassword), "changed",
 			  G_CALLBACK (field_changed_cb), config);
-	gtk_table_attach (GTK_TABLE (table), config->priv->wpassword, 1, 2, 6, 7,
+	gtk_table_attach (GTK_TABLE (table), config->priv->wpassword, 1, 2, 5, 6,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
+
+	exp = gtk_expander_new (_("Connection's specific parameters"));
+	config->priv->expander = exp;
+	gtk_widget_show (exp);
+	gtk_table_attach (GTK_TABLE (table), config->priv->expander, 0, 2, 6, 7,
+			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
+	config->priv->dsn_spec = gnome_db_dsn_spec_new (gnome_db_provider_selector_get_selected_provider 
+							(GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
+	g_signal_connect (G_OBJECT (config->priv->dsn_spec), "changed",
+			  G_CALLBACK (field_changed_cb), config);
+	gtk_container_add (GTK_CONTAINER (config->priv->expander), config->priv->dsn_spec);
+	gtk_widget_show (config->priv->dsn_spec);
 }
 
 static void
@@ -198,12 +221,12 @@ gnome_db_dsn_config_finalize (GObject *object)
 
 	/* free memory */
 	g_free (config->priv);
-	config->priv = NULL;
 
 	/* chain to parent class */
 	parent_class->finalize (object);
 }
 
+
 GType
 gnome_db_dsn_config_get_type (void)
 {
@@ -266,8 +289,9 @@ gnome_db_dsn_config_get_info (GnomeDbDsnConfig *config)
 			dsn_info->name[i] = '_';
 	}
 
-	dsn_info->provider = g_strdup (gnome_db_provider_selector_get_selected_provider (GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
-	dsn_info->cnc_string = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wstring)));
+	dsn_info->provider = g_strdup (gnome_db_provider_selector_get_selected_provider 
+				       (GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
+	dsn_info->cnc_string = gnome_db_dsn_spec_get_specs (GNOME_DB_DSN_SPEC (config->priv->dsn_spec));
 	dsn_info->description = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wdesc)));
 	dsn_info->username = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wuser)));
 	dsn_info->password = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wpassword)));
@@ -289,8 +313,9 @@ gnome_db_dsn_config_set_info (GnomeDbDsnConfig *config, const GdaDataSourceInfo
 	g_return_if_fail (dsn_info != NULL);
 
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wname), dsn_info->name);
-	gnome_db_provider_selector_set_selected_provider (GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider), dsn_info->provider);
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wstring), dsn_info->cnc_string ? dsn_info->cnc_string : "");
+	gnome_db_provider_selector_set_selected_provider (GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider), 
+							  dsn_info->provider);
+	gnome_db_dsn_spec_set_specs (GNOME_DB_DSN_SPEC (config->priv->dsn_spec), dsn_info->cnc_string);
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wdesc), dsn_info->description ? dsn_info->description : "");
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wuser), dsn_info->username ? dsn_info->username : "");
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wpassword), dsn_info->password ? dsn_info->password : "");
