@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1998-2002 The GNOME Foundation.
+ * Copyright (C) 1998 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -25,7 +25,7 @@
 #include <gtk/gtkwindow.h>
 #include <libgnomedb/gnome-db-error-dialog.h>
 #include <libgnomedb/gnome-db-util.h>
-#include "gnome-db-i18n.h"
+#include <glib/gi18n-lib.h>
 
 struct _GnomeDbErrorDialogPrivate {
 	GnomeDbError *error_widget;
@@ -179,7 +179,7 @@ gnome_db_error_dialog_finalize (GObject *object)
 
 /**
  * gnome_db_error_dialog_new
- * title: title of the dialog box
+ * @title: title of the dialog box
  *
  * Create a new GnomeDbErrorDialog, which is a subclass of the GnomeDialog
  * widget, just acting as a container for a GnomeDbError widget
@@ -195,11 +195,13 @@ gnome_db_error_dialog_new (gchar *title)
 	dialog = g_object_new (GNOME_DB_TYPE_ERROR_DIALOG, NULL);
 	if (title)
 		gnome_db_error_dialog_set_title (dialog, title);
-        icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
-        if (icon) {
-                gtk_window_set_icon (GTK_WINDOW (dialog), icon);
-                g_object_unref (icon);                                         
-        }                                                                      
+
+	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
+	if (icon) {
+		gtk_window_set_icon (GTK_WINDOW (dialog), icon);
+		g_object_unref (icon);
+	}
+
 	if (title)
 		gnome_db_error_dialog_set_title (dialog, title);
 
