@@ -123,7 +123,7 @@ enum
 	PROP_ACTIONS_VISIBLE
 };
 
-guint
+GType
 gnome_db_form_get_type (void)
 {
 	static GType type = 0;
@@ -223,7 +223,7 @@ gnome_db_form_new (void)
 
 /**
  * gnome_db_form_new_with_gda_model
- * @dict: a #GnomeDbDict dictionnary, or %NULL
+ * @dict: a #GnomeDbDict dictionary, or %NULL
  * @model: a #GdaDataModel
  *
  * Creates a new #GnomeDbForm widget suitable to display the data in @model
@@ -234,6 +234,7 @@ GtkWidget *
 gnome_db_form_new_with_gda_model (GnomeDbDict *dict, GdaDataModel *model)
 {
 	TO_IMPLEMENT;
+	return NULL;
 }
 
 
@@ -508,6 +509,18 @@ data_set_param_changed_cb (GnomeDbBasicForm *simple_form, GnomeDbParameter *para
 		g_assert (col >= 0);
 		gnome_db_data_proxy_set_value (form->priv->proxy, &iter, col, gnome_db_parameter_get_value (param));
 		modif_buttons_update (form);
+
+		/* REM:
+		 * if @form->priv->data_model is a GnomeDbResultSet, then we should also change the values
+		 * of @form->priv->proxy which are copies of values displayed in the GnomeDbEntryCombo
+		 * (use form->priv->data_set->data_set_data_for_params_source);
+		 * see GnomeDbGrid::data_cell_values_changed() for more information;
+		 *
+		 * OR
+		 * Add a "changed" signal to GnomeDbEntryCombo with the same arguments as the "changed" signal
+		 * of GnomeDbDataCellRendererCombo, and connect to that instead of monitoring the parameters change
+		 * for parameters constrained by value in a GnomeDbDataModel .
+		 */
 	}
 }
 
@@ -585,7 +598,7 @@ gnome_db_form_initialize (GnomeDbForm *form, GnomeDbQuery *orig_query, GtkWidget
 	realform = gnome_db_basic_form_new_in_layout (dict, form->priv->data_set, layout, box_widgets);
 	gtk_notebook_append_page (GTK_NOTEBOOK (nb), realform, NULL);
 	gtk_widget_show (realform);
-	if (!layout && gnome_db_data_proxy_is_read_only)
+	if (!layout && gnome_db_data_proxy_is_read_only (form->priv->proxy))
 		gnome_db_basic_form_show_entries_actions (GNOME_DB_BASIC_FORM (realform), FALSE);
 	form->priv->basic_form = realform;
 
@@ -773,6 +786,7 @@ modif_buttons_make (GnomeDbForm *form)
 	gtk_ui_manager_add_ui_from_string (ui, ui_actions_info, -1, NULL);
 	form->priv->uimanager = ui;
 	form->priv->modif_all = gtk_ui_manager_get_widget (ui, "/ToolBar");
+	g_object_set (G_OBJECT (form->priv->modif_all), "toolbar-style", GTK_TOOLBAR_ICONS, NULL);
 	gtk_widget_show (form->priv->modif_all);
 	gtk_box_pack_start (GTK_BOX (hbox), form->priv->modif_all, TRUE, TRUE, 0);
 
