@@ -86,7 +86,7 @@ GQuark gnome_db_graphviz_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_graphviz_get_type (void)
 {
 	static GType type = 0;
