@@ -163,7 +163,7 @@ enum
 	PROP_INFO_CELL_VISIBLE
 };
 
-guint
+GType
 gnome_db_grid_get_type (void)
 {
 	static GType type = 0;
