@@ -60,7 +60,7 @@ struct _GnomeDbGridClass
 /* 
  * Generic widget's methods 
  */
-guint             gnome_db_grid_get_type                  (void);
+GType             gnome_db_grid_get_type                  (void);
 
 GtkWidget        *gnome_db_grid_new                       (void);
 GtkWidget        *gnome_db_grid_new_with_gda_model        (GnomeDbDict *dict, GdaDataModel *model);
