@@ -121,7 +121,7 @@ GQuark gnome_db_join_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_join_get_type (void)
 {
 	static GType type = 0;
