@@ -130,7 +130,7 @@ GQuark gnome_db_model_wrapper_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_model_wrapper_get_type (void)
 {
 	static GType type = 0;
@@ -982,21 +982,26 @@ make_entity_fields (GnomeDbModelWrapper *wrapper)
 static gboolean
 gnome_db_wrapper_has_field (GnomeDbEntity *iface, GnomeDbField *field)
 {
-	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), FALSE);
-	g_return_val_if_fail (GNOME_DB_MODEL_WRAPPER (iface)->priv, FALSE);
+	GnomeDbModelWrapper *wrapper;
 
-	TO_IMPLEMENT;
-	return FALSE;
+	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), FALSE);
+	wrapper = GNOME_DB_MODEL_WRAPPER (iface);
+	g_return_val_if_fail (wrapper->priv, FALSE);
+	
+	return g_slist_find (wrapper->priv->fields, field) ? TRUE : FALSE;
 }
 
 static GSList *
 gnome_db_wrapper_get_fields (GnomeDbEntity *iface)
 {
-	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), NULL);
-	g_return_val_if_fail (GNOME_DB_MODEL_WRAPPER (iface)->priv, NULL);
+	GnomeDbModelWrapper *wrapper;
 
-	TO_IMPLEMENT;
-	return NULL;
+	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), NULL);
+	wrapper = GNOME_DB_MODEL_WRAPPER (iface);
+	g_return_val_if_fail (wrapper->priv, NULL);
+	make_entity_fields (wrapper);
+	
+	return g_slist_copy (wrapper->priv->fields);
 }
 
 static GnomeDbField *
@@ -1016,41 +1021,52 @@ gnome_db_wrapper_get_field_by_name (GnomeDbEntity *iface, const gchar *name)
 static GnomeDbField *
 gnome_db_wrapper_get_field_by_index (GnomeDbEntity *iface, gint index)
 {
+	GnomeDbModelWrapper *wrapper;
+
 	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), NULL);
-	g_return_val_if_fail (GNOME_DB_MODEL_WRAPPER (iface)->priv, NULL);
+	wrapper = GNOME_DB_MODEL_WRAPPER (iface);
+	g_return_val_if_fail (wrapper->priv, NULL);
 	g_return_val_if_fail (index >= 0, NULL);
-	g_return_val_if_fail (index < g_slist_length (GNOME_DB_MODEL_WRAPPER (iface)->priv->fields), NULL);
+	make_entity_fields (wrapper);
+	g_return_val_if_fail (index < g_slist_length (wrapper->priv->fields), NULL);
 	
-	TO_IMPLEMENT;
-	return NULL;
+	return GNOME_DB_FIELD (g_slist_nth_data (wrapper->priv->fields, index));
 }
 
 static gint
 gnome_db_wrapper_get_field_index (GnomeDbEntity *iface, GnomeDbField *field)
 {
-	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), -1);
-	g_return_val_if_fail (GNOME_DB_MODEL_WRAPPER (iface)->priv, -1);
+	GnomeDbModelWrapper *wrapper;
 
-	TO_IMPLEMENT;
-	return -1;
+	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), -1);
+	wrapper = GNOME_DB_MODEL_WRAPPER (iface);
+	g_return_val_if_fail (wrapper->priv, -1);
+	make_entity_fields (wrapper);
+
+	return g_slist_index (wrapper->priv->fields, field);
 }
 
 
 static gboolean
 gnome_db_wrapper_is_writable (GnomeDbEntity *iface)
 {
+	GnomeDbModelWrapper *wrapper;
+
 	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), FALSE);
-	g_return_val_if_fail (GNOME_DB_MODEL_WRAPPER (iface)->priv, FALSE);
-	
-	TO_IMPLEMENT;
+	wrapper = GNOME_DB_MODEL_WRAPPER (iface);
+	g_return_val_if_fail (wrapper->priv, FALSE);
+
 	return FALSE;
 }
 
 static GSList *
 gnome_db_wrapper_get_parameters (GnomeDbEntity *iface)
 {
+	GnomeDbModelWrapper *wrapper;
+
 	g_return_val_if_fail (iface && IS_GNOME_DB_MODEL_WRAPPER (iface), NULL);
-	g_return_val_if_fail (GNOME_DB_MODEL_WRAPPER (iface)->priv, NULL);
+	wrapper = GNOME_DB_MODEL_WRAPPER (iface);
+	g_return_val_if_fail (wrapper->priv, NULL);
 	
 	return NULL;
 }
