@@ -29,6 +29,7 @@
 #include "gnome-db-xml-storage.h"
 #include "gnome-db-referer.h"
 #include "gnome-db-data-model.h"
+#include "gnome-db-data-handler.h"
 
 /* 
  * Main static functions 
@@ -125,7 +126,7 @@ GQuark gnome_db_parameter_error_quark (void)
         return quark;
 }
 
-guint
+GType
 gnome_db_parameter_get_type (void)
 {
 	static GType type = 0;
@@ -207,7 +208,7 @@ gnome_db_parameter_init (GnomeDbParameter *parameter)
 	parameter->priv->change_with = NULL;
 
 	parameter->priv->invalid_forced = FALSE;
-	parameter->priv->valid = FALSE;
+	parameter->priv->valid = TRUE;
 	parameter->priv->default_forced = FALSE;
 	parameter->priv->value = NULL;
 	parameter->priv->default_value = NULL;
@@ -693,6 +694,46 @@ gnome_db_parameter_set_value (GnomeDbParameter *param, const GdaValue *value)
 	}
 }
 
+/**
+ * gnome_db_parameter_set_value_str
+ * @param: a #GnomeDbParameter object
+ * @value: a value to set the parameter to, as a string
+ *
+ * Same function as gnome_db_parameter_set_value() except that the value
+ * is provided as a string, and may return FALSE if the string did not
+ * represent a correct value for the data type of the parameter.
+ *
+ * Returns: TRUE if no error occured
+ */
+gboolean
+gnome_db_parameter_set_value_str (GnomeDbParameter *param, const gchar *value)
+{
+	g_return_val_if_fail (param && IS_GNOME_DB_PARAMETER (param), NULL);
+	g_return_val_if_fail (param->priv, NULL);
+
+	if (!value) {
+		gnome_db_parameter_set_value (param, NULL);	
+		return TRUE;
+	}
+	else {
+		GnomeDbDataHandler *dh;
+		GdaValue *gdaval;
+
+		dh = gnome_db_server_data_type_get_handler (param->priv->type);
+		gdaval = gnome_db_data_handler_get_value_from_str (dh, value,
+								   gnome_db_server_data_type_get_gda_type (param->priv->type));
+		if (!gdaval)
+			gdaval = gnome_db_data_handler_get_value_from_sql (dh, value,
+								   gnome_db_server_data_type_get_gda_type (param->priv->type));
+		if (gdaval) {
+			gnome_db_parameter_set_value (param, gdaval);
+			gda_value_free (gdaval);
+			return TRUE;
+		}
+		else
+			return FALSE;
+	}
+}
 
 /**
  * gnome_db_parameter_declare_invalid
@@ -1330,7 +1371,8 @@ gnome_db_parameter_dump (GnomeDbParameter *parameter, guint offset)
 	if (parameter->priv) {
 		GSList *list;
 		g_print ("%s" D_COL_H1 "GnomeDbParameter %p (%s), type=%s\n" D_COL_NOR, str, parameter,
-			 gnome_db_base_get_name (GNOME_DB_BASE (parameter)), gnome_db_base_get_name (GNOME_DB_BASE (parameter->priv->type)));
+			 gnome_db_base_get_name (GNOME_DB_BASE (parameter)), 
+			 gnome_db_base_get_name (GNOME_DB_BASE (parameter->priv->type)));
 		
 		list = parameter->priv->for_fields;
 		while (list) {
