@@ -124,7 +124,7 @@ GQuark gnome_db_qf_agg_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_qf_agg_get_type (void)
 {
 	static GType type = 0;
