@@ -123,7 +123,7 @@ GQuark gnome_db_qf_all_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_qf_all_get_type (void)
 {
 	static GType type = 0;
