@@ -62,7 +62,7 @@ struct _GnomeDbQfFuncClass
 	void   (*templ_signal)        (GnomeDbQfFunc *obj);
 };
 
-guint                  gnome_db_qf_func_get_type          (void);
+GType                  gnome_db_qf_func_get_type          (void);
 GObject               *gnome_db_qf_func_new_with_func     (GnomeDbQuery *query, GnomeDbServerFunction *func);
 GObject               *gnome_db_qf_func_new_with_xml_id   (GnomeDbQuery *query, const gchar *func_xml_id);
 
