@@ -84,8 +84,6 @@ static gboolean    gnome_db_qf_value_is_equal       (GnomeDbQfield *qfield1, Gno
 static void nullified_object_cb (GObject *obj, GnomeDbQfValue *field);
 
 static GSList   *gnome_db_qf_value_get_params (GnomeDbQfield *qfield);
-static gboolean  gnome_db_qf_value_restrict_with_field (GnomeDbQfValue *field, GnomeDbQfield *provider, GError **error);
-static gboolean  gnome_db_qf_value_restrict_with_field_xml (GnomeDbQfValue *field, const gchar *prov_xml_id, GError **error);
 
 static gboolean gnome_db_qf_value_render_find_value (GnomeDbQfValue *field, GnomeDbDataSet *context,
 					       const GdaValue **value_found, GnomeDbParameter **param_source);
@@ -143,7 +141,7 @@ GQuark gnome_db_qf_value_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_qf_value_get_type (void)
 {
 	static GType type = 0;
@@ -826,7 +824,7 @@ gnome_db_qf_value_is_value_null (GnomeDbQfValue *field, GnomeDbDataSet *context)
  *
  * Returns: TRUE if no error occurred
  */
-static gboolean
+gboolean
 gnome_db_qf_value_restrict_with_field (GnomeDbQfValue *field, GnomeDbQfield *provider, GError **error)
 {
 	g_return_val_if_fail (field && IS_GNOME_DB_QF_VALUE (field), FALSE);
@@ -894,7 +892,7 @@ gnome_db_qf_value_restrict_with_field (GnomeDbQfValue *field, GnomeDbQfield *pro
  *
  * Returns: TRUE if no error occurred
  */
-static gboolean 
+gboolean 
 gnome_db_qf_value_restrict_with_field_xml (GnomeDbQfValue *field, const gchar *prov_xml_id, GError **error)
 {
 	g_return_val_if_fail (field && IS_GNOME_DB_QF_VALUE (field), FALSE);
@@ -1211,8 +1209,17 @@ gnome_db_qf_value_load_from_xml (GnomeDbXmlStorage *iface, xmlNodePtr node, GErr
 
 	prop = xmlGetProp (node, "value");
 	if (prop) {
-		if (dh)
+		if (dh) {
 			field->priv->value = gnome_db_data_handler_get_value_from_str (dh, prop, field->priv->gda_type);
+			if (!field->priv->value) {
+				g_set_error (error,
+					     GNOME_DB_QF_VALUE_ERROR,
+					     GNOME_DB_QF_VALUE_XML_LOAD_ERROR,
+					     _("Can't interpret '%s' as a value"), prop);
+				g_free (prop);
+				return FALSE;
+			}
+		}
 
 		g_free (prop);
 	}
@@ -1393,8 +1400,17 @@ gnome_db_qf_value_render_as_sql (GnomeDbRenderer *iface, GnomeDbDataSet *context
 			if (param_source) {
 				gboolean use_default;
 				g_object_get (G_OBJECT (param_source), "use_default_value", &use_default, NULL);
-				if (use_default) 
-					str = g_strdup ("DEFAULT");
+				if (use_default) {
+					if (options & GNOME_DB_RENDERER_ERROR_IF_DEFAULT) {
+						g_set_error (error,
+							     GNOME_DB_QF_VALUE_ERROR,
+							     GNOME_DB_QF_VALUE_DEFAULT_PARAM_ERROR,
+							     "Default value requested");
+						return NULL;
+					}
+					else
+						str = g_strdup ("DEFAULT");
+				}
 			}
 			if (!str) {
 				if (value && (gda_value_get_type (value) != GDA_VALUE_TYPE_NULL)) {
