@@ -1,6 +1,6 @@
 /* gnome-db-qfield.c
  *
- * Copyright (C) 2003 - 2004  Vivien Malerba
+ * Copyright (C) 2003 - 2005  Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,6 +26,7 @@
 #include "gnome-db-qf-field.h"
 #include "gnome-db-qf-value.h"
 #include "gnome-db-qf-func.h"
+#include "gnome-db-qf-agg.h"
 #include "gnome-db-xml-storage.h"
 #include "gnome-db-ref-base.h"
 #include "gnome-db-server.h"
@@ -87,7 +88,7 @@ GQuark gnome_db_qfield_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_qfield_get_type (void)
 {
 	static GType type = 0;
@@ -210,7 +211,21 @@ gnome_db_qfield_new_from_xml (GnomeDbQuery *query, xmlNodePtr node, GError **err
 		}
 
 		if (!done && !strcmp (prop, "AGG")) {
-			TO_IMPLEMENT;
+			gchar *agg;
+
+			agg = xmlGetProp (node, "object");
+			if (agg) {
+				obj = gnome_db_qf_agg_new_with_xml_id (query, agg);
+				g_free (agg);
+			}
+			
+			if (!obj) {
+				g_set_error (error,
+					     GNOME_DB_QF_ALL_ERROR,
+					     GNOME_DB_QF_ALL_XML_LOAD_ERROR,
+					     _("Missing 'object' attribute in <GNOME_DB_QF>"));
+				return NULL;
+			}
 		}
 
 		if (!done && !strcmp (prop, "FUNC")) {
@@ -266,8 +281,10 @@ gnome_db_qfield_new_from_xml (GnomeDbQuery *query, xmlNodePtr node, GError **err
 
 		if (obj) {
 			attach_to_query (GNOME_DB_QFIELD (obj), query);
-			if (!gnome_db_xml_storage_load_from_xml (GNOME_DB_XML_STORAGE (obj), node, error))
+			if (!gnome_db_xml_storage_load_from_xml (GNOME_DB_XML_STORAGE (obj), node, error)) {
+				g_object_unref (obj);
 				return NULL;
+			}
 		}
 		else 
 			g_set_error (error,
