@@ -58,7 +58,7 @@ struct _GnomeDbResultSetClass
 	GnomeDbBaseClass             class;
 };
 
-guint           gnome_db_result_set_get_type           (void);
+GType           gnome_db_result_set_get_type           (void);
 
 GObject        *gnome_db_result_set_new                (GnomeDbQuery *query, GnomeDbTarget *modif_target);
 
