@@ -59,7 +59,7 @@ static gint gnome_db_selector_signals[LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class = NULL;
 
 
-guint
+GType
 gnome_db_selector_get_type (void)
 {
 	static GType type = 0;
@@ -358,7 +358,8 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 							    G_TYPE_POINTER,   /* OBJ_COLUMN */
 							    GDK_TYPE_PIXBUF,  /* PIXBUF_COLUMN */
 							    G_TYPE_INT,       /* CONTENTS_COLUMN */
-							    G_TYPE_POINTER)); /* SUB_MODULE_COLUMN */
+							    G_TYPE_POINTER,   /* SUB_MODULE_COLUMN */
+							    G_TYPE_BOOLEAN)); /* ERROR_COLUMN marker column */
 		mgsel->priv->model = model;
 
 		/* 'top level' modules selection (not using ref_object) */
@@ -468,6 +469,9 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 		renderer = gtk_cell_renderer_pixbuf_new ();
 		gtk_tree_view_column_pack_start (column, renderer, FALSE);
 		renderer = gtk_cell_renderer_text_new ();
+		g_object_set (G_OBJECT (renderer), 
+			      "strikethrough", TRUE,
+			      "strikethrough-set", FALSE, NULL);
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
 		gtk_tree_view_column_set_cell_data_func (column,
 							 renderer,
@@ -527,7 +531,12 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 				case G_TYPE_STRING:
 				default:
 					renderer = gtk_cell_renderer_text_new ();
-					column = gtk_tree_view_column_new_with_attributes (str, renderer, "text", i, NULL);
+					g_object_set (G_OBJECT (renderer), 
+						      "strikethrough", TRUE,
+						      "strikethrough-set", FALSE, NULL);
+					column = gtk_tree_view_column_new_with_attributes (str, renderer, 
+											   "text", i, 
+											   "strikethrough-set", ERROR_COLUMN, NULL);
 					break;
 				case G_TYPE_BOOLEAN:
 					renderer = gtk_cell_renderer_toggle_new ();
@@ -573,13 +582,14 @@ tree_value_set_func (GtkTreeViewColumn *tree_column,
         GdkPixbuf *pixbuf;
         gchar *text;
         GList *renderers;
+	gboolean error;
 
-        gtk_tree_model_get (model, iter, PIXBUF_COLUMN, &pixbuf, NAME_COLUMN, &text, -1);
+        gtk_tree_model_get (model, iter, PIXBUF_COLUMN, &pixbuf, NAME_COLUMN, &text, ERROR_COLUMN, &error, -1);
         renderers = gtk_tree_view_column_get_cell_renderers (tree_column);
         cell = renderers->data;
         g_object_set (G_OBJECT (cell), "pixbuf", pixbuf, NULL);
         cell = renderers->next->data;
-        g_object_set (G_OBJECT (cell), "text", text, NULL);
+        g_object_set (G_OBJECT (cell), "text", text, "strikethrough-set", error, NULL);
         g_free (text);
         g_list_free (renderers);
 }
