@@ -93,7 +93,7 @@ GQuark gnome_db_server_aggregate_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_server_aggregate_get_type (void)
 {
 	static GType type = 0;
