@@ -60,7 +60,7 @@ struct _GnomeDbServerDataTypeClass
 	void   (*templ_signal)        (GnomeDbServerDataType *obj);
 };
 
-guint               gnome_db_server_data_type_get_type      (void);
+GType               gnome_db_server_data_type_get_type      (void);
 GObject            *gnome_db_server_data_type_new           (GnomeDbServer *srv);
 
 void                gnome_db_server_data_type_set_sqlname   (GnomeDbServerDataType *dt, const gchar *sqlname);
