@@ -1,6 +1,6 @@
 /* gnome-db-server-function.c
  *
- * Copyright (C) 2003 - 2004 Vivien Malerba
+ * Copyright (C) 2003 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -93,7 +93,7 @@ GQuark gnome_db_server_function_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_server_function_get_type (void)
 {
 	static GType type = 0;
@@ -448,17 +448,12 @@ gnome_db_function_load_from_xml (GnomeDbXmlStorage *iface, xmlNodePtr node, GErr
 			prop = xmlGetProp (subnode, "type");
 			if (prop) {
 				dt = gnome_db_server_get_data_type_by_xml_id (func->priv->srv, prop);
+				if (!dt) {
+					/* Add a new custom data type (this can't happen at the moment because of the DTD) */
+					TO_IMPLEMENT;
+				}
 				g_free (prop);
 			}
-
-			/* if (!dt) { */
-/* 				g_set_error (error, */
-/* 					     GNOME_DB_SERVER_FUNCTION_ERROR, */
-/* 					     GNOME_DB_SERVER_FUNCTION_XML_LOAD_ERROR, */
-/* 					     _("Can't find data type for function '%s'"),  */
-/* 					     gnome_db_base_get_name (GNOME_DB_BASE (func))); */
-/* 				return FALSE; */
-/* 			} */
 			
 			prop = xmlGetProp (subnode, "way");
 			if (prop) {
