@@ -61,7 +61,7 @@ static void hist_item_free (HistItem *item);
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
 
-guint
+GType
 gnome_db_sql_console_get_type (void)
 {
 	static GType type = 0;
