@@ -117,7 +117,7 @@ GQuark gnome_db_table_field_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_table_field_get_type (void)
 {
 	static GType type = 0;
