@@ -130,7 +130,7 @@ GQuark gnome_db_table_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_table_get_type (void)
 {
 	static GType type = 0;
