@@ -107,7 +107,7 @@ GQuark gnome_db_target_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_target_get_type (void)
 {
 	static GType type = 0;
