@@ -101,7 +101,7 @@ struct _GnomeDbCanvasDbRelationsPrivate
 	GnomeDbDatabase  *db;
 };
 
-guint
+GType
 gnome_db_canvas_db_relations_get_type (void)
 {
 	static GType type = 0;
