@@ -1,6 +1,6 @@
 /* gnome-db-canvas-item.c
  *
- * Copyright (C) 2002 - 2004 Vivien Malerba
+ * Copyright (C) 2002 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  * USA
  */
 
+#include <gtk/gtk.h>
 #include "marshal.h"
 #include "gnome-db-decl.h"
 #include "gnome-db-canvas-item.h"
@@ -73,7 +74,7 @@ static gint gnome_db_canvas_item_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *base_parent_class = NULL;
 
-guint
+GType
 gnome_db_canvas_item_get_type (void)
 {
 	static GType type = 0;
