@@ -75,7 +75,7 @@ struct _GnomeDbCanvasItemClass
 	void (*extra_event)  (GnomeDbCanvasItem * citem, GdkEventType event_type);
 };
 
-guint        gnome_db_canvas_item_get_type (void);
+GType        gnome_db_canvas_item_get_type (void);
 
 GnomeDbCanvas    *gnome_db_canvas_item_get_canvas     (GnomeDbCanvasItem *item);
 GnomeDbGraphItem *gnome_db_canvas_item_get_graph_item (GnomeDbCanvasItem *item);
