@@ -1,6 +1,6 @@
 /* gnome-db-canvas-join.c
  *
- * Copyright (C) 2004 Vivien Malerba
+ * Copyright (C) 2004 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  * USA
  */
 
+#include <gtk/gtk.h>
 #include "gnome-db-canvas.h"
 #include "gnome-db-canvas-join.h"
 #include "gnome-db-canvas-entity.h"
@@ -64,7 +65,7 @@ struct _GnomeDbCanvasJoinPrivate
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
 
-guint
+GType
 gnome_db_canvas_join_get_type (void)
 {
 	static GType type = 0;
@@ -488,11 +489,11 @@ popup_join_properties_cb (GtkMenuItem *mitem, GnomeDbCanvasJoin *cc)
 	GtkWidget *rb, *rb2, *entry, *button;
 	gchar *str;
 	
-	dlg = gtk_dialog_new_with_buttons (_("Join properties"), top /*GTK_WINDOW (top)*/, 0,
+	dlg = gtk_dialog_new_with_buttons (_("Join properties"), (GtkWindow *) top /*GTK_WINDOW (top)*/, 0,
 					   GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					   NULL);
 	vbox = GTK_DIALOG (dlg)->vbox;
-	gtk_box_set_spacing (GTK_BOX (vbox), GNOME_PAD);
+	gtk_box_set_spacing (GTK_BOX (vbox), 6);
 
 	/*
 	 * join type
@@ -502,7 +503,7 @@ popup_join_properties_cb (GtkMenuItem *mitem, GnomeDbCanvasJoin *cc)
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
 	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 6);
 	gtk_widget_show (label);
 
 	hbox = gtk_hbox_new (FALSE, 0); /* HIG */
@@ -513,7 +514,7 @@ popup_join_properties_cb (GtkMenuItem *mitem, GnomeDbCanvasJoin *cc)
 	gtk_widget_show (label);
 
 	table = gtk_table_new (5, 2, FALSE);
-	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 	gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
 	gtk_widget_show (table);
 
@@ -568,7 +569,7 @@ popup_join_properties_cb (GtkMenuItem *mitem, GnomeDbCanvasJoin *cc)
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
 	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 6);
 	gtk_widget_show (label);
 	g_object_set_data (G_OBJECT (dlg), "join_cond_label", label);
 
