@@ -19,6 +19,7 @@
  * USA
  */
 
+#include <gtk/gtk.h>
 #include "marshal.h"
 #include <libgnomedb/gnome-db-entity.h>
 #include <libgnomedb/gnome-db-query.h>
@@ -68,7 +69,7 @@ struct _GnomeDbCanvasQueryStructPrivate
 	GHashTable  *hash_joins; /* key = GnomeDbJoin, value = GnomeDbCanvasJoin */
 };
 
-guint
+GType
 gnome_db_canvas_query_struct_get_type (void)
 {
 	static GType type = 0;
