@@ -1,6 +1,6 @@
 /* gnome-db-graph.c
  *
- * Copyright (C) 2004 Vivien Malerba
+ * Copyright (C) 2004 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -100,7 +100,7 @@ GQuark gnome_db_graph_error_quark (void)
 }
 
 
-guint
+GType
 gnome_db_graph_get_type (void)
 {
 	static GType type = 0;
@@ -235,17 +235,17 @@ gnome_db_graph_new (GnomeDbDict *dict, GnomeDbGraphType type)
 	GnomeDbGraph *graph;
 	guint id;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 
-	obj = g_object_new (GNOME_DB_GRAPH_TYPE, "dict", dict, NULL);
+	obj = g_object_new (GNOME_DB_GRAPH_TYPE, "dict", ASSERT_DICT (dict), NULL);
 	graph = GNOME_DB_GRAPH (obj);
 
-	g_object_get (G_OBJECT (dict), "graph_serial", &id, NULL);
+	g_object_get (G_OBJECT (ASSERT_DICT (dict)), "graph_serial", &id, NULL);
 	gnome_db_base_set_id (GNOME_DB_BASE (obj), id);
 
 	graph->priv->type = type;
 
-	gnome_db_dict_declare_graph (dict, graph);
+	gnome_db_dict_declare_graph (ASSERT_DICT (dict), graph);
 
 	return obj;
 }
