@@ -88,7 +88,7 @@ gnome_db_entry_time_cell_editable_init (GtkCellEditableIface *iface)
 	iface->start_editing = gnome_db_entry_time_start_editing;
 }
 
-guint
+GType
 gnome_db_entry_time_get_type (void)
 {
 	static GType type = 0;
