@@ -95,7 +95,7 @@ struct  _GnomeDbEntryWrapperPriv {
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
 
-guint
+GType
 gnome_db_entry_wrapper_get_type (void)
 {
 	static GType type = 0;
