@@ -1,6 +1,6 @@
 /* gnome-db-handler-time.c
  *
- * Copyright (C) 2003 Vivien Malerba
+ * Copyright (C) 2003 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,13 +45,13 @@ static void gnome_db_handler_compute_locale (GnomeDbHandlerTime *hdl);
 /* GnomeDbDataHandler interface */
 static void         gnome_db_handler_time_data_handler_init      (GnomeDbDataHandlerIface *iface);
 static GnomeDbDataEntry *gnome_db_handler_time_get_entry_from_value   (GnomeDbDataHandler *dh, const GdaValue *value,
-							    GdaValueType type);
+								       GdaValueType type);
 static gchar       *gnome_db_handler_time_get_sql_from_value     (GnomeDbDataHandler *dh, const GdaValue *value);
 static gchar       *gnome_db_handler_time_get_str_from_value     (GnomeDbDataHandler *dh, const GdaValue *value);
 static GdaValue    *gnome_db_handler_time_get_value_from_sql     (GnomeDbDataHandler *dh, const gchar *sql,
-							    GdaValueType type);
+								  GdaValueType type);
 static GdaValue    *gnome_db_handler_time_get_value_from_str     (GnomeDbDataHandler *dh, const gchar *sql,
-							    GdaValueType type);
+								  GdaValueType type);
 
 static GdaValue    *gnome_db_handler_time_get_sane_init_value    (GnomeDbDataHandler * dh, GdaValueType type);
 
@@ -79,6 +79,7 @@ static gint gnome_db_handler_time_signals[LAST_SIGNAL] = { };
 typedef struct _LocaleSetting {
 	GDateDMY        dmy_order[3];
 	gboolean        twodigit_years;
+	gint            current_offset; /* 1900, 2000, etc... */
 	gchar           separator;	
 } LocaleSetting;
 
@@ -96,7 +97,7 @@ struct  _GnomeDbHandlerTimePriv {
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
 
-guint
+GType
 gnome_db_handler_time_get_type (void)
 {
 	static GType type = 0;
@@ -177,6 +178,7 @@ gnome_db_handler_time_init (GnomeDbHandlerTime *hdl)
 	hdl->priv->sql_locale->dmy_order[1] = G_DATE_DAY;
 	hdl->priv->sql_locale->dmy_order[2] = G_DATE_YEAR;
 	hdl->priv->sql_locale->twodigit_years = FALSE;
+	hdl->priv->sql_locale->current_offset = 0;
 	hdl->priv->sql_locale->separator = '-';
 
 	hdl->priv->str_locale = g_new0 (LocaleSetting, 1);
@@ -184,6 +186,7 @@ gnome_db_handler_time_init (GnomeDbHandlerTime *hdl)
 	hdl->priv->str_locale->dmy_order[1] = G_DATE_MONTH;
 	hdl->priv->str_locale->dmy_order[2] = G_DATE_YEAR;
 	hdl->priv->str_locale->twodigit_years = FALSE;
+	hdl->priv->sql_locale->current_offset = 0;
 	hdl->priv->str_locale->separator = '/';
 
 	gnome_db_handler_compute_locale (hdl);
@@ -299,6 +302,9 @@ gnome_db_handler_compute_locale (GnomeDbHandlerTime *hdl)
 		gchar *strings[3];
 #endif
 		gint i;
+		time_t now;
+		struct tm *now_tm;
+
 		for (i=0; i<3; i++) {
 			switch (nums[i]) {
 			case 7:
@@ -316,6 +322,11 @@ gnome_db_handler_compute_locale (GnomeDbHandlerTime *hdl)
 				break;
 			}
 		}
+		
+		now = time (NULL);
+		now_tm = localtime (&now);
+		hdl->priv->str_locale->current_offset = ((now_tm->tm_year + 1900) / 100) * 100;
+
 #ifdef debug		
 		for (i=0; i<3; i++) {
 			switch (hdl->priv->str_locale->dmy_order[i]) {
@@ -337,7 +348,7 @@ gnome_db_handler_compute_locale (GnomeDbHandlerTime *hdl)
 		g_print ("\tlocale order = %s %s %s, separator = %c\n", 
 			 strings[0], strings[1], strings[2], hdl->priv->str_locale->separator);
 		if (hdl->priv->str_locale->twodigit_years)
-			g_print ("\tlocale has 2 digits year\n");
+			g_print ("\tlocale has 2 digits year, using %d as offset\n", hdl->priv->str_locale->current_offset);
 		else
 			g_print ("\tlocale has 4 digits year\n");
 #endif
@@ -502,8 +513,12 @@ render_date_locale (const GdaDate *date, LocaleSetting *locale)
 			g_string_append_printf (string, "%02d", date->month);
 			break;
 		case G_DATE_YEAR:
-			if (locale->twodigit_years)
-				g_string_append_printf (string, "%02d", date->year - 1900);
+			if (locale->twodigit_years) {
+				if ((date->year >= locale->current_offset) && (date->year < locale->current_offset + 100))
+					g_string_append_printf (string, "%02d", date->year - locale->current_offset);
+				else
+					g_string_append_printf (string, "%04d", date->year);
+			}
 			else
 				g_string_append_printf (string, "%04d", date->year);
 			break;
@@ -569,7 +584,7 @@ static gboolean make_date (GnomeDbHandlerTime *hdl, GdaDate *date, const gchar *
 static gboolean make_time (GnomeDbHandlerTime *hdl, GdaTime *timegda, const gchar *value);
 static GdaValue *
 gnome_db_handler_time_get_value_from_locale (GnomeDbDataHandler *iface, const gchar *sql, 
-				       GdaValueType type, LocaleSetting *locale)
+					     GdaValueType type, LocaleSetting *locale)
 {
 	GnomeDbHandlerTime *hdl;
 	GdaValue *value = NULL;
@@ -702,6 +717,8 @@ make_date (GnomeDbHandlerTime *hdl, GdaDate *date, const gchar *value, LocaleSet
 				break;
 			case G_DATE_YEAR:
 				date->year = nums[i];
+				if (date->year < 100)
+					date->year += locale->current_offset;
 				break;
 			}
 		
