@@ -457,7 +457,7 @@ build_menu (GtkWidget *mainwin, GtkWidget *top_nb)
 	menubar1 = gtk_menu_bar_new ();
 	gtk_widget_show (menubar1);
 
-	menuitem1 = gtk_menu_item_new_with_mnemonic (_("_Fichier"));
+	menuitem1 = gtk_menu_item_new_with_mnemonic (_("_File"));
 	gtk_widget_show (menuitem1);
 	gtk_container_add (GTK_CONTAINER (menubar1), menuitem1);
 	
